@@ -18,7 +18,7 @@
 
 static int im_uim_fd = -1;
 static int read_tag;
-static char *result = NULL;
+static guint snooper_id;
 
 struct preedit_segment {
   int attr;
@@ -34,6 +34,7 @@ struct candidate_window {
   int layout_begin;
   int is_active;
   int select_count;
+  gboolean left;
   GdkRectangle cursor;
 };
 
@@ -51,7 +52,6 @@ typedef struct _IMUIMContext {
   GdkWindow *win;
   GdkWindow *toplevel;
   GdkRectangle preedit_pos; /* preedit_pos not always point the cursor location */
-  int mode;
   /**/
   struct _IMUIMContext *prev, *next;
 } IMUIMContext;
@@ -75,6 +75,7 @@ static GdkFilterReturn
 toplevel_window_candidate_cb (GdkXEvent *xevent, GdkEvent *ev, gpointer data);
 
 static void layout_candidate(IMUIMContext *uic);
+static void update_candidate(IMUIMContext *uic);
 
 static void
 im_uim_parse_helper_str(char *str);
@@ -108,6 +109,7 @@ void
 im_uim_commit_string(void *ptr, char *str)
 {
   IMUIMContext *uic = (IMUIMContext *)ptr;
+  g_return_if_fail(str);
   g_signal_emit_by_name(uic, "commit", str);
 }
 
@@ -128,13 +130,15 @@ static void
 pushback_cb(void *ptr, int attr, char *str)
 {
   IMUIMContext *uic = ptr;
-
-  if(!str)
+  g_return_if_fail(str);
+  
+  if(!strcmp(str,"") && attr != UPeAttr_Cursor)
     return;
+
   uic->pseg = realloc(uic->pseg,
 		      sizeof(struct preedit_segment) *
-		     (uic->nr_psegs + 1));
-  uic->pseg[uic->nr_psegs].str = strdup(str);
+		      (uic->nr_psegs + 1));
+  uic->pseg[uic->nr_psegs].str = g_strdup(str);
   uic->pseg[uic->nr_psegs].attr = attr;
   uic->nr_psegs ++;
 }
@@ -143,20 +147,10 @@ static void
 update_cb(void *ptr)
 {
   IMUIMContext *uic = ptr;
+  g_return_if_fail(ptr);
   g_signal_emit_by_name(uic, "preedit_changed");
 }
 
-static void
-mode_cb(void *ptr, int mode)
-{
-  IMUIMContext *uic = ptr;
-  if (uic->mode == mode) {
-    return ;
-  }
-  gtk_option_menu_set_history(GTK_OPTION_MENU(uic->menu), mode);
-  uic->mode = mode;
-}
-
 static int
 convert_keyval(int key)
 {
@@ -188,6 +182,9 @@ convert_keyval(int key)
   case GDK_F11: return UKey_F11;
   case GDK_F12: return UKey_F12;
   case GDK_Multi_key: return UKey_Multi_key;
+  case GDK_Mode_switch: return UKey_Mode_switch;
+  case GDK_Henkan_Mode: return UKey_Henkan_Mode;
+  case GDK_Muhenkan: return UKey_Muhenkan;
   case GDK_Shift_L: return UKey_Shift_key;
   case GDK_Shift_R: return UKey_Shift_key;
   case GDK_Control_L: return UKey_Control_key;
@@ -231,20 +228,8 @@ static gboolean
 filter_keypress(GtkIMContext *ic,
 		GdkEventKey *key)
 {
-  int rv;
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
-  int kv = convert_keyval(key->keyval);
-  int mod = convert_modifier(key->state);
-
-  if (key->type == GDK_KEY_RELEASE) {
-    rv = uim_release_key(uic->uc, kv, mod);
-  } else {
-    rv = uim_press_key(uic->uc, kv, mod);
-  }
-  if (rv) {
-    return gtk_im_context_filter_keypress(uic->slave, key);
-  }
-  return TRUE;
+  return gtk_im_context_filter_keypress(uic->slave, key);
 }
 
 static char *
@@ -293,7 +278,7 @@ im_uim_get_preedit_string(GtkIMContext *ic, gchar **str,
   if (attrs) {
     *attrs = pango_attr_list_new();
   }
-  tmp = strdup("");
+  tmp = g_strdup("");
   for (i = 0; i < uic->nr_psegs; i++) {
     if (uic->pseg[i].attr & UPeAttr_Cursor) {
       pos = g_utf8_strlen(tmp,-1);
@@ -319,13 +304,18 @@ im_uim_set_cursor_location(GtkIMContext *ic,
 			   GdkRectangle *area)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
+
+  if(uic->cwin.left &&  uic->nr_psegs > 0){
+    return;
+  }
+
   uic->cwin.cursor.x= area->x;
   uic->cwin.cursor.y= area->y;
   uic->cwin.cursor.height= area->height;
   uic->cwin.cursor.width= area->width;
 
   uic->preedit_pos.x = area->x;
-  uic->preedit_pos.x = area->x;
+  uic->preedit_pos.y = area->y;
 }
 
 
@@ -334,6 +324,7 @@ im_uim_commit_cb(GtkIMContext *ic,
 		 const gchar *str,
 		 IMUIMContext *is)
 {
+  g_return_if_fail(str);
   g_signal_emit_by_name(is, "commit", str);
 }
 
@@ -352,14 +343,8 @@ update_status_window(IMUIMContext *uic)
 }
 
 static void
-focus_in(GtkIMContext *ic)
+check_helper_connection()
 {
-  IMUIMContext *uic = IM_UIM_CONTEXT(ic);
-  IMUIMContext *cc;
-
-  focused_context = uic;
-
-
   if(im_uim_fd < 0) {
     im_uim_fd     = uim_helper_init_client_fd(im_uim_helper_disconnect_cb);
     if (im_uim_fd >= 0) {
@@ -367,6 +352,17 @@ focus_in(GtkIMContext *ic)
 			       helper_read_cb, 0);
     }
   }
+}
+
+static void
+focus_in(GtkIMContext *ic)
+{
+  IMUIMContext *uic = IM_UIM_CONTEXT(ic);
+  IMUIMContext *cc;
+
+  focused_context = uic;
+
+  check_helper_connection();
 
   uim_prop_list_update(uic->uc);
 
@@ -385,11 +381,10 @@ focus_in(GtkIMContext *ic)
     gdk_window_add_filter(uic->toplevel,
 			  toplevel_window_candidate_cb,
 			  uic);
-       
-    }
-
+  }
+  
   uim_helper_client_focus_in(uic->uc);
-
+  
   update_status_window(uic);
   if (uic->stat_win) {
     gtk_widget_show(uic->stat_win);
@@ -400,6 +395,9 @@ focus_out(GtkIMContext *ic)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
 
+  if(focused_context == uic)
+    focused_context = NULL;
+
   if (uic->win && uic->stat_win) {
     gtk_widget_hide(uic->stat_win);
   }
@@ -421,7 +419,7 @@ void im_uim_reset(GtkIMContext *ic)
 
   uim_reset_context(uic->uc);
 
-  g_warning (_("reset called: implement me"));
+  //  g_warning (_("reset called: implement me"));
 
 }
 
@@ -435,8 +433,8 @@ set_use_preedit(GtkIMContext *ic, gboolean use_preedit)
     preedit_label = gtk_label_new("");
     gtk_container_add(GTK_CONTAINER(preedit_window), preedit_label);
     
-    g_signal_connect ( G_OBJECT(ic), "preedit-changed",
-		       G_CALLBACK (show_preedit), preedit_label );
+    g_signal_connect(G_OBJECT(ic), "preedit-changed",
+		     G_CALLBACK(show_preedit), preedit_label );
     gtk_widget_show_all(preedit_window);
   }
 }
@@ -489,30 +487,13 @@ show_preedit(GtkIMContext *ic, GtkWidget *preedit_label)
   
 }
 
-/*
-static GdkFilterReturn
-resize_window_cb (GdkXEvent *xevent, GdkEvent *ev, gpointer data)
-{
-  IMUIMContext *uic;
-
-  if(!data)
-    return GDK_FILTER_CONTINUE;   
-
-  uic = IM_UIM_CONTEXT(data);
-
-
-  return GDK_FILTER_CONTINUE;
-
-}
-*/
-
 static GdkFilterReturn
 toplevel_window_candidate_cb (GdkXEvent *xevent, GdkEvent *ev, gpointer data)
 {
-  IMUIMContext *uic;
-
-  uic = IM_UIM_CONTEXT(data);
+  IMUIMContext *uic = focused_context;
 
+  if(!uic)
+    return GDK_FILTER_CONTINUE;
 
   if(uic->cwin.top_win && uic->cwin.is_active ){
     layout_candidate(uic);
@@ -526,14 +507,24 @@ static void
 set_client_window(GtkIMContext *ic, GdkWindow *w)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
-
   GdkWindow *toplevel = NULL;
 
-  if(w)
+  if(w) {
     toplevel = gdk_window_get_toplevel(w);
+  }
+
+  if(uic->toplevel != toplevel) {
+    
+    if(toplevel) {
+      g_object_ref(toplevel);
+    }
+    if(uic->toplevel) {
+      g_object_unref(uic->toplevel);
+    }
+    uic->toplevel = toplevel;
+  }
 
   uic->win = w;
-  uic->toplevel = toplevel;
   update_status_window(uic);
 }
 
@@ -545,7 +536,7 @@ static void
 im_uim_finalize(GObject *obj)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(obj);
-  set_client_window(GTK_IM_CONTEXT(uic), NULL);
+  //  set_client_window(GTK_IM_CONTEXT(uic), NULL);
 
   uic->next->prev = uic->prev;
   uic->prev->next = uic->next;
@@ -569,6 +560,9 @@ im_uim_finalize(GObject *obj)
 				       uic);
   g_object_unref(uic->slave);
   parent_class->finalize (obj);
+
+  if(uic == focused_context)
+    focused_context = NULL;
 }
 
 static void
@@ -588,70 +582,6 @@ im_uim_class_init (GtkIMContextClass *class)
   object_class->finalize = im_uim_finalize;
 }
 
-static gint
-select_mode(GtkWidget *w, gpointer p)
-{
-  IMUIMContext *is = p;
-  int nth = gtk_option_menu_get_history(GTK_OPTION_MENU(is->menu));
-  uim_set_mode(is->uc, nth);
-  return 0;
-  }
-
-static void
-update_status_menu(IMUIMContext *uic)
-{
-  int i, nr;
-  GtkWidget *menu;
-  GtkWidget *hbox;
-
-  nr = uim_get_nr_modes(uic->uc);
-  if (!nr) {
-    uic->menu = NULL;
-    uic->stat_win = NULL;
-    return ;
-  }
-
-  /* menu */
-  uic->menu = gtk_option_menu_new();
-  uic->stat_win = gtk_window_new(GTK_WINDOW_POPUP);
-  menu = gtk_menu_new();
-  for (i = 0; i < nr; i++) {
-    GtkWidget *item;
-    char *name;
-    name = uim_get_mode_name(uic->uc, i);
-    item = gtk_menu_item_new_with_label(name);
-    gtk_menu_append(menu, item);
-    gtk_signal_connect(GTK_OBJECT(item), "activate",
-		       GTK_SIGNAL_FUNC(select_mode), uic);
-    gtk_widget_show(item);
-  }
-
-  hbox = gtk_hbox_new(FALSE, 0);
-
-  gtk_container_add(GTK_CONTAINER(uic->stat_win), hbox);
-  gtk_option_menu_set_menu(GTK_OPTION_MENU(uic->menu), menu);
-  gtk_box_pack_start(GTK_BOX(hbox), uic->menu, FALSE, FALSE, 2);
-  gtk_option_menu_set_history(GTK_OPTION_MENU(uic->menu),
-			      uim_get_current_mode(uic->uc));
-  gtk_widget_show(uic->menu);
-  gtk_widget_show(hbox);
-}
-
-static void
-update_mode_list_cb(void *ptr)
-{
-  IMUIMContext *uic = ptr;
-  if (uic->menu) {
-    gtk_widget_destroy(uic->menu);
-  }
-  if (uic->stat_win) {
-    gtk_widget_destroy(uic->stat_win);
-  }
-  uic->menu = NULL;
-  uic->stat_win = NULL;
-  update_status_menu(uic);
-}
-
 static void
 update_prop_list_cb(void *ptr, char *str)
 {
@@ -702,10 +632,19 @@ init_candidate_win(IMUIMContext *uic)
   struct candidate_window *cwin = &uic->cwin;
   GtkWidget *vbox;
   GtkWidget *numlabel;
+  gchar *leftp;
 
   if (cwin->top_win) {
     return ;
   }
+
+  leftp = uim_symbol_value_str("candidate-window-position");
+ 
+  if(leftp && !strcmp(leftp,"left")) {
+    cwin->left = TRUE;
+  } else {
+    cwin->left = FALSE;
+  }
   cwin->top_win = gtk_window_new(GTK_WINDOW_POPUP);
   cwin->clist = gtk_clist_new(2);
   cwin->is_active = 0;
@@ -719,8 +658,6 @@ init_candidate_win(IMUIMContext *uic)
   gtk_box_pack_start(GTK_BOX(vbox), cwin->clist, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), numlabel, FALSE, FALSE, 0);
 
-  gtk_window_set_default_size(cwin->top_win, 80, 0);
-
   gtk_container_add(GTK_CONTAINER(cwin->top_win), vbox);
   {
     gint x,y,w,h,d;
@@ -729,8 +666,6 @@ init_candidate_win(IMUIMContext *uic)
     gtk_window_move(GTK_WINDOW(cwin->top_win), x , y+h);
   }
 
-
-
   gtk_clist_set_column_auto_resize( GTK_CLIST(cwin->clist), 0, TRUE );
   gtk_clist_set_column_auto_resize( GTK_CLIST(cwin->clist), 1, TRUE );
 
@@ -761,24 +696,40 @@ trim_index(IMUIMContext *uic)
   return changed;
 }
 
+static void
+calc_candidate_window_size(struct candidate_window cwin, int *width, int *height)
+{
+  GtkRequisition req_clist;
+  GtkRequisition req_numlabel;
+  gtk_widget_size_request(GTK_WIDGET(cwin.clist), &req_clist);
+  gtk_widget_size_request(GTK_WIDGET(cwin.numlabel), &req_numlabel);
+
+  *height = req_clist.height + req_numlabel.height;
+  *width  = req_clist.width;
+
+  if(*width < 80)
+    *width = 80;
+
+  gtk_window_resize(GTK_WINDOW(cwin.top_win), *width, *height);
+}
+
 static void
 layout_candidate(IMUIMContext *uic)
 {
-  char *buf[2];
-  char idx[20];
   int  i, x, y, topwin_x, topwin_y;
   int  sc_he, cw_he; /*screen height, candidate window height*/
   int  sc_wi, cw_wi;
-  char *new_first_cand = NULL;
-  char *old_first_cand = NULL;
+  char idx[20];
+  char *new_cand = NULL;
+  char *old_cand = NULL;
 
   trim_index(uic);
 
+  calc_candidate_window_size(uic->cwin, &cw_wi, &cw_he);
+
   sc_he = gdk_screen_get_height(gdk_screen_get_default ());
   sc_wi = gdk_screen_get_width (gdk_screen_get_default ());
 
-  gtk_window_get_size(GTK_WINDOW(uic->cwin.top_win), &cw_wi, &cw_he);
-
   gdk_window_get_origin(uic->win, &topwin_x, &topwin_y);
 
   if(sc_wi <  topwin_x + uic->cwin.cursor.x + cw_wi ) {
@@ -794,49 +745,62 @@ layout_candidate(IMUIMContext *uic)
   }
 
   gtk_window_move(GTK_WINDOW(uic->cwin.top_win), x, y );
+  
+  for (i = 0; i < NR_CANDIDATES; i++) {
+    if (i < uic->cwin.nr_candidates - uic->cwin.layout_begin) {
+      new_cand = uim_get_candidate(uic->uc, uic->cwin.layout_begin + i);
+      gtk_clist_get_text(GTK_CLIST(uic->cwin.clist), i, 1, &old_cand);
 
-  new_first_cand = uim_get_candidate(uic->uc, uic->cwin.layout_begin);
-  gtk_clist_get_text(GTK_CLIST(uic->cwin.clist), 0, 1, &old_first_cand);
+      g_return_if_fail(new_cand);
 
-  if(!new_first_cand)
-    return;
-
-  /* This condition works almost case, but not perfect.
-     We must think out more good way.  */
-  if( !old_first_cand ||
-      strcmp( old_first_cand, new_first_cand) !=0 ) {
-
-    gtk_clist_freeze(GTK_CLIST(uic->cwin.clist));
-    gtk_clist_clear(GTK_CLIST(uic->cwin.clist));
-
-    for (i = 0; i < NR_CANDIDATES; i++) {
-      if (i < uic->cwin.nr_candidates - uic->cwin.layout_begin) {
-	g_sprintf(idx, "%2d", i + uic->cwin.layout_begin + 1 );
-	buf[0] = idx;
-	buf[1] = uim_get_candidate(uic->uc, i + uic->cwin.layout_begin);
-      } else {
-	buf[0] = "";
-	buf[1] = "";
+      if(!old_cand || strcmp(new_cand, old_cand) != 0) {
+	update_candidate(uic);
+	free(new_cand);
+	break;
       }
-      gtk_clist_append(GTK_CLIST(uic->cwin.clist), buf);
-
-      if(buf[1] && strcmp(buf[1],"") != 0 )
-	free(buf[1]);
-   
+      free(new_cand);
     }
-    gtk_clist_thaw( GTK_CLIST(uic->cwin.clist) );
   }
   
-  free( new_first_cand );
 
+  if(uic->cwin.candidate_index >= 0) {
   uic->cwin.select_count ++;
   gtk_clist_select_row(GTK_CLIST(uic->cwin.clist),
 		       uic->cwin.candidate_index - uic->cwin.layout_begin,
 		       0);
-  sprintf(idx, "%d / %d",uic->cwin.candidate_index + 1 , uic->cwin.nr_candidates); 
+  }
 
+  sprintf(idx, "%d / %d",uic->cwin.candidate_index + 1 , uic->cwin.nr_candidates);  
   gtk_label_set_text( GTK_LABEL(uic->cwin.numlabel), idx);
+}
 
+
+static void
+update_candidate(IMUIMContext *uic)
+{
+  char *buf[2];
+  char idx[20];
+  int i;
+
+  gtk_clist_freeze(GTK_CLIST(uic->cwin.clist));
+  gtk_clist_clear(GTK_CLIST(uic->cwin.clist));
+  
+  for (i = 0; i < NR_CANDIDATES; i++) {
+    if (i < uic->cwin.nr_candidates - uic->cwin.layout_begin) {
+      g_sprintf(idx, "%2d", i + uic->cwin.layout_begin + 1 );
+      buf[0] = idx;
+      buf[1] = uim_get_candidate(uic->uc, i + uic->cwin.layout_begin);
+    } else {
+      buf[0] = "";
+      buf[1] = "";
+    }
+    gtk_clist_append(GTK_CLIST(uic->cwin.clist), buf);
+    
+    if(buf[1] && strcmp(buf[1],"") != 0 )
+      free(buf[1]);
+  }
+  gtk_clist_thaw( GTK_CLIST(uic->cwin.clist) );
+  
 }
 
 static void
@@ -846,7 +810,7 @@ cand_begin_cb(void *ptr, int nr, int index)
   init_candidate_win(uic);
   uic->cwin.is_active = 1;
   uic->cwin.nr_candidates = nr;
-  uic->cwin.candidate_index = index;
+  uic->cwin.candidate_index = -1;
   uic->cwin.layout_begin = 0;
   layout_candidate(uic);
   gtk_widget_show(uic->cwin.top_win);
@@ -867,6 +831,7 @@ cand_end_cb(void *ptr)
   if(uic->cwin.top_win) {
     gtk_widget_hide(uic->cwin.top_win);
   }
+  uic->nr_psegs = 0;
   uic->cwin.is_active = 0;
 }
 
@@ -901,20 +866,12 @@ im_module_create (const gchar *context_id)
     return NULL;
   }
 
-  if(im_uim_fd < 0) {
-    im_uim_fd     = uim_helper_init_client_fd(im_uim_helper_disconnect_cb);
-    if (im_uim_fd >= 0) {
-      read_tag = gdk_input_add(im_uim_fd, (GdkInputCondition)GDK_INPUT_READ,
-			       helper_read_cb, 0);
-    }
-  }
+  check_helper_connection();
 
   /**/
-  uim_set_mode_cb(uic->uc, mode_cb);
+
   uim_set_preedit_cb(uic->uc, clear_cb,
 		     pushback_cb, update_cb);
-  uim_set_mode_list_update_cb(uic->uc,
-			      update_mode_list_cb);
   uim_set_prop_list_update_cb(uic->uc,
 			      update_prop_list_cb);
   uim_set_prop_label_update_cb(uic->uc,
@@ -926,12 +883,13 @@ im_module_create (const gchar *context_id)
   
   uim_prop_list_update(uic->uc);
 
-  uic->mode = uim_get_current_mode(uic->uc);
+
+  /* slave exists for using gtk+'s table based input method */
   uic->slave = g_object_new(GTK_TYPE_IM_CONTEXT_SIMPLE,
 			    NULL);
   g_signal_connect(G_OBJECT(uic->slave), "commit",
 		   G_CALLBACK(im_uim_commit_cb), uic);
-  update_status_menu(uic);
+
   /**/
   uic->next = context_list.next;
   uic->prev = (IMUIMContext *)&context_list;
@@ -959,15 +917,24 @@ im_module_list(GtkIMContextInfo ***contexts,
   info_list = malloc(sizeof(GtkIMContextInfo*)*nr);
   for (i = 0; i < nr; i++) {
     char *name = uim_get_im_name(uc, i);
-    char *s = alloca(strlen(name) + 5);
-    sprintf(s, "uim-%s", name);
-    free(name);
+    char *lang = uim_get_im_language(uc, i);
+    char s[100];
+
     info_list[i] = malloc(sizeof(GtkIMContextInfo));
-    info_list[i]->context_id = strdup(s);
-    info_list[i]->context_name = strdup(s);
+
+
+    sprintf(s, "uim-%s", name);
+    info_list[i]->context_id = g_strdup(s);
+
+    sprintf(s, "uim-%s (%s)", name, lang);
+    info_list[i]->context_name = g_strdup(s);
+
     info_list[i]->domain = "gtk+";
     info_list[i]->domain_dirname = "";
-    info_list[i]->default_locales = uim_get_im_language(uc, i);
+    info_list[i]->default_locales = g_strdup(lang);
+
+    g_free(name);
+    g_free(lang);
   }
   uim_release_context(uc);
   *contexts = info_list;
@@ -977,33 +944,23 @@ im_module_list(GtkIMContextInfo ***contexts,
 static void
 im_uim_parse_helper_str(char *str)
 {
-  gchar **tmp;
-  gchar **tmp2;
-  int i = 0;
-
-  tmp = g_strsplit(str, "\n\n", 0);
-
-  while(tmp[i]){
+  gchar **lines;
   
-    if(focused_context) {
-      if(strcmp("prop_list_get", tmp[i]) == 0){
-	uim_prop_list_update(focused_context->uc);
-      } else if(strcmp("prop_label_get", tmp[i]) == 0) {
-	uim_prop_label_update(focused_context->uc);
-      } else if(strncmp("prop_activate", tmp[i], 13) == 0) {
-	tmp2 = g_strsplit(str, "\n", 0);
-	if(tmp2 && tmp2[0]) {
-	  uim_prop_activate(focused_context->uc, tmp2[1]);
-	  g_strfreev(tmp2);
-	}
-      } else if(strcmp("focus_in", tmp[i]) == 0) {
-	focused_context = NULL;
+  if(focused_context) {
+    if(strncmp("prop_list_get", str, 13) == 0){
+      uim_prop_list_update(focused_context->uc);
+    } else if(strncmp("prop_label_get", str, 14) == 0) {
+      uim_prop_label_update(focused_context->uc);
+    } else if(strncmp("prop_activate", str, 13) == 0) {
+      lines = g_strsplit(str, "\n", 0);
+      if(lines && lines[0]) {
+	uim_prop_activate(focused_context->uc, lines[1]);
+	g_strfreev(lines);
       }
+    } else if(strncmp("focus_in", str, 8) == 0) {
+      focused_context = NULL;
     }
-    i++;
   }
-  
-  g_strfreev(tmp);
 }
 
 
@@ -1012,19 +969,10 @@ helper_read_cb(gpointer p, int fd, GdkInputCondition c)
 {
   char *tmp;
 
-  tmp = uim_helper_read_proc(fd);
-
-  if(!result)
-    result = strdup("");
-
-  result = (char*)realloc(result, strlen(result) + strlen(tmp)+1);
-  strcat(result, tmp);
-  free(tmp);
-
-  if(uim_helper_str_terminated(result)) {
-    im_uim_parse_helper_str(result);
-    free(result);
-    result = strdup("");
+  uim_helper_read_proc(fd);
+  while ((tmp = uim_helper_get_message())) {
+    im_uim_parse_helper_str(tmp);
+    free(tmp);
   }
 }
 
@@ -1035,6 +983,28 @@ im_uim_helper_disconnect_cb(void)
   gdk_input_remove(read_tag);
 }
 
+/* XXX:This is not recommended way!! */
+gboolean
+uim_key_snoop(GtkWidget *grab_widget, GdkEventKey *key, gpointer data)
+{
+  if(focused_context){
+    int rv;
+    int kv = convert_keyval(key->keyval);
+    int mod = convert_modifier(key->state);
+    
+    if (key->type == GDK_KEY_RELEASE) {
+      rv = uim_release_key(focused_context->uc, kv, mod);
+    } else {
+      rv = uim_press_key(focused_context->uc, kv, mod);
+    }
+    if (rv) {
+      return FALSE;
+    }
+    return TRUE;
+  }
+  return FALSE;
+}
+
 void
 im_module_init(GTypeModule *type_module)
 {
@@ -1046,10 +1016,17 @@ im_module_init(GTypeModule *type_module)
 				"GtkIMContextUIM",
 				&class_info, 0);
 
-  
+  /* XXX:This is not recommended way!! */
+  snooper_id = gtk_key_snooper_install( (GtkKeySnoopFunc*)uim_key_snoop, NULL );
+
 }
 
 void
 im_module_exit()
 {
+  if (im_uim_fd != -1) {
+    uim_helper_close_client_fd(im_uim_fd);
+  }
+  gtk_key_snooper_remove(snooper_id);
+  uim_quit();
 }
