@@ -4,6 +4,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <glib/gprintf.h>
+#include <stdlib.h>
 #include <string.h>
 
 #define NR_CANDIDATES 10 /* this should be configurable*/
@@ -160,10 +161,12 @@ candwin_update(char **str)
 {
   int index;
   char idx[20];
+  GtkTreePath* path;
+
   sscanf(str[1],"%d",&index);
   cwin.candidate_index = index;
 
-  GtkTreePath* path = gtk_tree_path_new_from_indices(index, -1);
+  path = gtk_tree_path_new_from_indices(index, -1);
   gtk_tree_view_set_cursor(GTK_TREE_VIEW(m_view),
 			   path, NULL, FALSE);
   gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(m_view),
@@ -234,13 +237,15 @@ read_cb(gpointer p, int fd, GdkInputCondition c)
   char buf[1024];
 
   while (fgets (buf, sizeof(buf), stdin ) != NULL ) {
-    if(strcmp( buf, "\n" ) == 0)
-      break;
+    if(strcmp( buf, "\n" ) == 0) {
+       str_parse(tmp);
+       free(tmp);
+       return;
+    }
     tmp = realloc(tmp, strlen(tmp) + strlen(buf) + 1);
     strcat(tmp, buf);
   }
-  str_parse(tmp);
-  free(tmp);
+  exit(-1);
 }
 
 int main (int argc, char *argv[])
@@ -253,4 +258,5 @@ int main (int argc, char *argv[])
   read_tag = gdk_input_add(0, (GdkInputCondition)GDK_INPUT_READ,
 			   read_cb, 0);
   gtk_main ();
+  return 0;
 }
