@@ -14,13 +14,27 @@
      (and (= (to-lower-char key) 106)
 	  (control-key-mask key-state))
      (generic-on-key key key-state))))
-
-(define anthy-off-key
+(define anthy-latin-key
   (lambda (key key-state)
-    (or
-	 (= (to-lower-char key) 108)
+    (or 
+     (and 
+      (= (to-lower-char key) 108)
+      (not (modifier-key-mask key-state)))
      (generic-on-key key key-state))))
-
+(define anthy-wide-latin-key
+  (lambda (key key-state)
+    (and (= (to-lower-char key) 108)
+	 (shift-key-mask key-state))))
+(define anthy-kana-toggle-key
+  (lambda (key key-state)
+    (and
+     (= (to-lower-char key) 113)
+     (not (modifier-key-mask key-state)))))
+(define anthy-commit-as-opposite-kana-key
+  (lambda (key key-state)
+    (and
+     (= (to-lower-char key) 113)
+     (shift-key-mask key-state))))
 (define anthy-begin-conv-key
   (lambda (key key-state)
     (= (to-lower-char key) 32)))
@@ -60,12 +74,24 @@
 (define anthy-delete-key
   (lambda (key key-state)
     (generic-delete-key key key-state)))
+(define anthy-kill-key
+  (lambda (key key-state)
+    (generic-kill-key key key-state)))
+(define anthy-kill-backward-key
+  (lambda (key key-state)
+    (generic-kill-backward-key key key-state)))
 (define anthy-go-left-key
   (lambda (key key-state)
     (generic-go-left-key key key-state)))
 (define anthy-go-right-key
   (lambda (key key-state)
     (generic-go-right-key key key-state)))
+(define anthy-beginning-of-preedit-key
+  (lambda (key key-state)
+    (generic-beginning-of-preedit-key key key-state)))
+(define anthy-end-of-preedit-key
+  (lambda (key key-state)
+    (generic-end-of-preedit-key key key-state)))
 (define anthy-next-segment-key
   (lambda (key key-state)
     (generic-go-right-key key key-state)))
@@ -153,13 +179,31 @@
 (define anthy-context-set-candidate-op-count!
   (lambda (ac c)
     (set-car! (nthcdr 9 ac) c)))
+
+(define anthy-context-wide-latin
+  (lambda (ac)
+    (car (nthcdr 10 ac))))
+
+(define anthy-context-set-wide-latin!
+  (lambda (ac c)
+    (set-car! (nthcdr 10 ac) c)))
+
+(define anthy-context-kana-mode
+  (lambda (ac)
+    (car (nthcdr 11 ac))))
+
+(define anthy-context-set-kana-mode!
+  (lambda (ac c)
+    (set-car! (nthcdr 11 ac) c)))
+
+
 ;; on/off state(compose,input) anthy-context-id string rkc index-list
 ;; cur-seg candidate-window
 (define anthy-context-new
  (lambda ()
    (let ((c '())
 	 (rkc (rk-context-new ja-rk-rule #t #f)))
-     (set! c (copy-list '(() () () () () () () () () () ())))
+     (set! c (copy-list '(() () () () () () () () () () () #f #t)))
      (anthy-context-set-ac-id!
       c
       (if anthy-init-lib-ok?
@@ -170,16 +214,34 @@
      (anthy-context-set-on! c #f)
      c)))
 
+(define anthy-context-kana-toggle
+  (lambda (ac)
+    (anthy-context-set-kana-mode!
+     ac
+     (not (anthy-context-kana-mode ac)))))
+
 (define anthy-make-string
-  (lambda (sl dir)
+  (lambda (sl dir kana)
     (if sl
 	(if dir
-	    (string-append (anthy-make-string (cdr sl) dir)
-			   (caar sl))
-	    (string-append (caar sl)
-			   (anthy-make-string (cdr sl) dir)))
+	    (string-append (anthy-make-string (cdr sl) dir kana)
+			   (if kana
+			       (cdar sl)
+			       (caar sl)))
+	    (string-append (if kana
+			       (cdar sl)
+			       (caar sl))
+			   (anthy-make-string (cdr sl) dir kana)))
 	"")))
 
+(define anthy-make-left-string
+  (lambda (sl kana)
+    (anthy-make-string sl #t kana)))
+
+(define anthy-make-right-string
+  (lambda (sl kana)
+    (anthy-make-string sl #f kana)))
+
 (define anthy-get-candidate
   (lambda (ac nth-seg)
     (let ((id (anthy-context-ac-id ac))
@@ -192,8 +254,10 @@
       (set-context-data! c
 			 (anthy-context-new))
       (im-clear-mode-list id)
-      (im-pushback-mode-list id "RAW")
+      (im-pushback-mode-list id "ľ")
       (im-pushback-mode-list id "Ҥ餬")
+      (im-pushback-mode-list id "")
+      (im-pushback-mode-list id "ѱѿ")
       (im-update-mode-list id)
       (im-update-mode id 0)
       (anthy-update-prop-list id)
@@ -211,9 +275,19 @@
     (let* ((c (find-context id))
 	   (ac (context-data c)))
       (anthy-flush ac)
-      (if (= mode 0)
-	  (anthy-context-set-on! ac #f)
-	  (anthy-context-set-on! ac #t))
+      (cond  ; `case' is not supported by uim
+       ((= mode 0)  ; 'direct'
+        (anthy-context-set-on! ac #f)
+        (anthy-context-set-wide-latin! ac #f))
+       ((= mode 1)  ; 'hiragana'
+        (anthy-context-set-on! ac #t)
+        (anthy-context-set-kana-mode! ac #f))
+       ((= mode 2)  ; 'katakana'
+        (anthy-context-set-on! ac #t)
+        (anthy-context-set-kana-mode! ac #t))
+       ((= mode 3)  ; 'wide-latin'
+        (anthy-context-set-on! ac #f)
+        (anthy-context-set-wide-latin! ac #t))) 
       (anthy-update-preedit ac id))
     ()))
 
@@ -251,7 +325,7 @@
       (if (anthy-on-key key key-state)
 	  (begin
 	    (anthy-begin-input ac)
-	    (im-update-mode id 1)
+	    (anthy-update-mode ac id)
 	    (anthy-update-prop-label ac id))
 	  (im-commit-raw (context-id c))))))
 
@@ -288,6 +362,7 @@
   (lambda (ac)
     (let* ((ac-id (anthy-context-ac-id ac))
 	   (rkc (anthy-context-rkc ac))
+	   (kana (anthy-context-kana-mode ac))
 	   (last "")
 	   (res))
       ;; get resudiual 'n'
@@ -298,8 +373,8 @@
       (anthy-lib-set-string
        ac-id
        (string-append
-		(anthy-make-string (anthy-context-left-string ac) #t)
-		(anthy-make-string (anthy-context-right-string ac) #f)))
+		(anthy-make-left-string (anthy-context-left-string ac) #f)
+		(anthy-make-right-string (anthy-context-right-string ac) #f)))
       (anthy-context-set-index-list!
        ac
        (anthy-make-index-list
@@ -315,11 +390,22 @@
 	((rkc (anthy-context-rkc ac))
 	 (direct (ja-direct (charcode->string key))))
       (and
-       (if (anthy-off-key key key-state)
+
+       (if (anthy-wide-latin-key key key-state)
 	   (begin
 	     (anthy-flush ac)
 	     (anthy-context-set-on! ac #f)
-	     (im-update-mode id 0)
+	     (anthy-context-set-wide-latin! ac #t)
+	     (anthy-update-mode ac id)
+	     (anthy-update-prop-label ac id)
+	     #f)
+	   #t)
+       (if (anthy-latin-key key key-state)
+	   (begin
+	     (anthy-flush ac)
+	     (anthy-context-set-on! ac #f)
+	     (anthy-context-set-wide-latin! ac #f)
+	     (anthy-update-mode ac id)
 	     (anthy-update-prop-label ac id)
 	     #f)
 	   #t)
@@ -333,6 +419,13 @@
 	     (im-commit-raw id)
 	     #f)
 	   #t)
+     (if (anthy-kana-toggle-key key key-state)
+	 (begin 
+	   (anthy-context-kana-toggle ac)
+	   (anthy-update-mode ac id)
+	   (anthy-update-prop-label ac id)
+	   #f)
+	 #t)
        ;; modifiers (except shift) => ignore
        (if (and (modifier-key-mask key-state)
 		(not (shift-key-mask key-state)))
@@ -365,7 +458,8 @@
 
 (define anthy-proc-input-state-with-preedit
   (lambda (ac id key key-state)
-    (let ((rkc (anthy-context-rkc ac)))
+    (let ((rkc (anthy-context-rkc ac))
+	  (kana (anthy-context-kana-mode ac)))
       (and
        ;; begin conversion
        (if (and
@@ -395,6 +489,48 @@
 		      (cdr (anthy-context-right-string ac)))))
 	     #f)
 	   #t)
+       ;; kill
+       (if (anthy-kill-key key key-state)
+	   (begin
+	     (anthy-context-set-right-string! ac ())
+	     #f)
+	   #t)
+       ;; kill-backward
+       (if (anthy-kill-backward-key key key-state)
+	   (begin
+	     (rk-flush rkc)
+	     (anthy-context-set-left-string! ac ())
+	     #f)
+	   #t)
+       ;; Ҥ餬ʥ⡼ɤǥʤꤹ
+       (if (anthy-commit-as-opposite-kana-key key key-state)
+	   (begin
+	     (im-commit
+	      id
+	      (string-append
+	       (anthy-make-left-string (anthy-context-left-string ac)
+				       (not kana))
+	       (anthy-make-right-string (anthy-context-right-string ac)
+					(not kana))))
+	     (anthy-flush ac)
+	     #f)
+	   #t)
+       ;; ߤΤʤ塢Ҥ餬/ʥ⡼ɤڤ괹
+       (if (anthy-kana-toggle-key key key-state)
+	   (begin
+	     (im-commit
+	      id
+	      (string-append
+	       (anthy-make-left-string (anthy-context-left-string ac)
+				       kana)
+	       (anthy-make-right-string (anthy-context-right-string ac)
+					kana)))
+	     (anthy-flush ac)
+	     (anthy-context-kana-toggle ac)
+	     (anthy-update-mode ac id)
+	     (anthy-update-prop-label ac id)
+	     #f)
+	   #t)
        ;; cancel
        (if (anthy-cancel-key key key-state)
 	   (begin
@@ -407,8 +543,8 @@
 	     (im-commit
 	      id
 	      (string-append
-	       (anthy-make-string (anthy-context-left-string ac) #t)
-	       (anthy-make-string (anthy-context-right-string ac) #f)))
+		(anthy-make-left-string (anthy-context-left-string ac) kana)
+		(anthy-make-right-string (anthy-context-right-string ac) kana)))
 	     (anthy-flush ac)
 	     #f)
 	   #t)
@@ -436,6 +572,32 @@
 		   (anthy-append-string ac c)))
 	     #f)
 	   #t)
+       ;; beginning-of-preedit
+       (if (anthy-beginning-of-preedit-key key key-state)
+	   (begin
+	     (if (anthy-context-left-string ac)
+		 (begin
+		   (anthy-context-set-right-string!
+		    ac
+		    (append
+		     (reverse (anthy-context-left-string ac))
+		     (anthy-context-right-string ac)))
+		   (anthy-context-set-left-string! ac ())))
+	     #f)
+	   #t)
+       ;; end-of-preedit
+       (if (anthy-end-of-preedit-key key key-state)
+	   (begin
+	     (if (anthy-context-right-string ac)
+		 (begin
+		   (anthy-context-set-left-string!
+		    ac
+		    (append
+		     (reverse (anthy-context-right-string ac))
+		     (anthy-context-left-string ac)))
+		   (anthy-context-set-right-string! ac ())))
+	     #f)
+	   #t)
        ;; modifiers (except shift) => ignore
        (if (and (modifier-key-mask key-state)
 		(not (shift-key-mask key-state)))
@@ -482,20 +644,21 @@
 
 (define anthy-input-state-preedit
   (lambda (ac id)
-    (let ((rkc (anthy-context-rkc ac)))
+    (let ((rkc (anthy-context-rkc ac))
+	  (kana (anthy-context-kana-mode ac)))
       (im-clear-preedit id)
       (im-pushback-preedit
        id preedit-underline
-       (anthy-make-string (anthy-context-left-string ac) #t))
+       (anthy-make-left-string (anthy-context-left-string ac) kana))
       (im-pushback-preedit id preedit-underline
                            (rk-pending rkc))
       (if (anthy-has-preedit? ac)
 	  (im-pushback-preedit id preedit-cursor ""))
       (im-pushback-preedit
        id preedit-underline
-       (anthy-make-string (anthy-context-right-string ac) #f))
+       (anthy-make-right-string (anthy-context-right-string ac) kana))
       (im-update-preedit id))))
-
+  
 (define anthy-get-commit-string
   (lambda (ac idx nseg)
     (let ((ac-id (anthy-context-ac-id ac)))
@@ -642,7 +805,8 @@
 	   (anthy-reset-candidate-window ac id)
 	   #f)
 	 #t)
-     (if (modifier-key-mask key-state)
+     (if (and (modifier-key-mask key-state)
+	      (not (shift-key-mask key-state)))
 	 #f
 	 #t)
      (begin
@@ -650,6 +814,31 @@
        (anthy-proc-input-state ac id key key-state)))
     ()))
 
+
+(define anthy-proc-wide-latin
+  (lambda (c key key-state)
+    (let* ((w (ja-wide (charcode->string key)))
+	   (id (context-id c))
+	   (ac (context-data c)))
+      (and
+       (if (anthy-on-key key key-state)
+	   (begin
+	     (anthy-flush ac)
+	     (anthy-update-mode ac id)
+	     (anthy-update-prop-label ac id)
+	     #f)
+	   #t)
+       (if (and (modifier-key-mask key-state)
+		(not (shift-key-mask key-state)))
+	   (begin
+	     (im-commit-raw id)
+	     #f)
+	   #t)
+       (if w
+	   (im-commit id w)
+	   (im-commit-raw id)))
+      ())))
+
 (define anthy-press-key-handler
   (lambda (id key key-state)
     (let* ((c (find-context id))
@@ -658,7 +847,9 @@
 	  (if (anthy-context-state ac)
 	      (anthy-proc-compose-state ac id key key-state)
 	      (anthy-proc-input-state ac id key key-state))
-	  (anthy-proc-raw-state c key key-state))
+	  (if (anthy-context-wide-latin ac)
+	      (anthy-proc-wide-latin c key key-state)
+	      (anthy-proc-raw-state c key key-state)))
       ;; preedit
       (anthy-update-preedit ac id))))
 
@@ -703,30 +894,51 @@
     (if (string-equal? message
 		       "prop_anthy_hiragana")
 	(begin
-	  (im-update-mode id 1)
-	  (anthy-context-set-on! ac #t)))
+	  (anthy-context-set-on! ac #t)
+	  (anthy-context-set-kana-mode! ac #t)))
     (if (string-equal? message
 		       "prop_anthy_katakana")
 	(begin
-	  (im-update-mode id 1)
-	  (anthy-context-set-on! ac #t)))
+	  (anthy-context-set-on! ac #t)
+	  (anthy-context-set-kana-mode! ac #f)))
     (if (string-equal? message
 		       "prop_anthy_direct")
 	(begin
-	  (im-update-mode id 0)
 	  (anthy-context-set-on! ac #f)))
-    (anthy-update-prop-label ac id))))
+	  (anthy-context-set-wide-latin! ac #f)))
+    (if (string-equal? message
+		       "prop_anthy_zenkaku")
+	(begin
+	  (anthy-context-set-on! ac #f)
+	  (anthy-context-set-wide-latin! ac #t)))
+    (anthy-update-mode ac id)
+    (anthy-update-prop-label ac id))
 
 (define anthy-update-prop-label
   (lambda (ac id)
     (let ((str ""))
       (set! str
 	    (if (anthy-context-on ac)
-		"\tҤ餬\n"
-		"a\tľ\n"))
+		(if (anthy-context-kana-mode ac)
+		    "\t\n"
+		    "\tҤ餬\n")
+		(if (anthy-context-wide-latin ac)
+		    "\tѱѿ\n"
+		    "a\tľ\n")))
       (set! str (string-append str "\tϥ⡼\n"))
       (im-update-prop-label id str))))
-  
+
+(define anthy-update-mode
+  (lambda (ac id)
+    (if (anthy-context-on ac)
+	(if (anthy-context-kana-mode ac)
+	    (im-update-mode id 2)
+	    (im-update-mode id 1))
+	(if (anthy-context-wide-latin ac)
+	    (im-update-mode id 3)
+	    (im-update-mode id 0)))
+    (anthy-update-prop-label ac id)))
+
 (define anthy-prop-list
   '(
     ("branch" "Ͼ")
@@ -745,14 +957,17 @@
 	   (ac (context-data c))
 	   (str "branch\t"))
       (set! str (string-append str
-	    (if (anthy-context-on ac)
-		"\tҤ餬\n"
-		"a\tľ\n")))
+			       (if (anthy-context-on ac)
+				   "\tҤ餬\n"
+				   (if (anthy-context-wide-latin ac)
+				       "\tѱѿ\n"
+				       "a\tľ\n"))))
       (set! str (string-append
 		 str
 		 "leaf\t\tҤ餬\tҤ餬ʤǤ\tprop_anthy_hiragana\n"
 		 "leaf\t\t\tʤϤǤޤ\tprop_anthy_katakana\n"
 		 "leaf\tA\tľ\t쥯ȤǤ\tprop_anthy_direct\n"
+		 "leaf\t\tѱѿ\tѱѿ⡼\tprop_anthy_zenkaku\n"
 		 "branch\t\t\nleaf\t\t޻\tdesc\tprop_anthy_roma\n"
 		 "leaf\t\t\t\tdescription\tprop_anthy_kana\n"))
       (im-update-prop-list id str)
