@@ -11,6 +11,13 @@
 (define preedit-underline 1)
 (define preedit-reverse 2)
 (define preedit-cursor 4)
+(define preedit-attr?
+  (lambda (attr)
+    (or
+     (= attr preedit-none)
+     (= attr preedit-underline)
+     (= attr preedit-reverse)
+     (= attr preedit-cursor))))
 
 ;; config
 (define enable-im-switch #f)
