@@ -161,9 +161,9 @@
     ((("t" "s" "u"). ())("".""))
     ((("x" "t" "u"). ())("".""))
     ((("x" "t" "s" "u"). ())("".""))
-    ((("c"). ("c"))("".""))
+    ((("c" "c"). ("c"))("".""))
 
-    ((("d"). ("d"))("".""))
+    ((("d" "d"). ("d"))("".""))
 
     ((("d" "a"). ())("".""))
     ((("d" "i"). ())("".""))
@@ -255,7 +255,7 @@
     ((("p" "y" "e"). ())("Ԥ"."ԥ"))
     ((("p" "y" "o"). ())("Ԥ"."ԥ"))
 
-    ((("m" "m"). ())("".""))
+    ((("m" "m"). ("m"))("".""))
     ((("m" "b"). ("b"))("".""))
     ((("m" "p"). ("p"))("".""))
 
@@ -271,6 +271,7 @@
     ((("m" "y" "e"). ())("ߤ"."ߥ"))
     ((("m" "y" "o"). ())("ߤ"."ߥ"))
 
+    ((("y" "y"). ("y"))("".""))
     ((("y" "a"). ())("".""))
     ((("y" "u"). ())("".""))
     ((("y" "o"). ())("".""))
