@@ -16,7 +16,6 @@
 (require "japanese.scm")
 (require "generic-key.scm")
 
-
 (define skk-dic-file-name "/usr/share/skk/SKK-JISYO.L")
 (define skk-personal-dic-filename
   (string-append (getenv "HOME") "/.skk-jisyo"))
@@ -26,13 +25,15 @@
 ;; configs
 (define skk-use-candidate-window? #t)
 (define skk-candidate-op-count 0)
-(define skk-use-recursive-learning? #f)
+(define skk-use-recursive-learning? #t)
+(define skk-style 'skk-style-ddskk-like)
 ;; key defs
 (define skk-latin-key
   (lambda (key key-state)
     (or
      (generic-off-key key key-state)
-     (= (to-lower-char key) 108))))
+     (and (= (to-lower-char key) 108)
+	  (not (modifier-key-mask key-state))))))
 (define skk-wide-latin-key
   (lambda (key key-state)
     (and (= (to-lower-char key) 108)
@@ -46,9 +47,13 @@
      (and (= (to-lower-char key) 106)
 	  (control-key-mask key-state))
      (generic-on-key key key-state))))
+(define skk-return-key
+  (lambda (key key-state)
+    (generic-return-key key key-state)))
 (define skk-commit-key
   (lambda (key key-state)
-    (generic-commit-key key key-state)))
+    (and (= (to-lower-char key) 106)
+	 (control-key-mask key-state))))
 (define skk-next-candidate-key
   (lambda (key key-state)
     (generic-next-candidate-key key key-state)))
@@ -60,7 +65,9 @@
 
 (define skk-kana-toggle-key
   (lambda (key key-state)
-    (= (to-lower-char key) 113)))
+    (and
+     (= (to-lower-char key) 113)
+     (not (modifier-key-mask key-state)))))
 (define skk-cancel-key
   (lambda (key key-state)
     (generic-cancel-key key key-state)))
@@ -77,6 +84,69 @@
   (lambda (key key-state)
     (= key 81)))
 
+;; style elements
+(define skk-preedit-attr-mode-mark)
+(define skk-preedit-attr-head)
+(define skk-preedit-attr-okuri)
+(define skk-preedit-attr-pending-rk)
+(define skk-preedit-attr-conv-body)
+(define skk-preedit-attr-conv-okuri)
+(define skk-preedit-attr-direct-pending-rk)
+(define skk-preedit-attr-child-beginning-mark)
+(define skk-preedit-attr-child-end-mark)
+(define skk-preedit-attr-child-committed)
+(define skk-child-context-beginning-mark)
+(define skk-child-context-end-mark)
+(define skk-show-cursor-on-preedit?)
+(define skk-show-candidates-with-okuri?)
+;; style specification
+(define skk-style-spec
+  '(;; (style-element-name . validator)
+    (skk-preedit-attr-mode-mark            . preedit-attr?)
+    (skk-preedit-attr-head                 . preedit-attr?)
+    (skk-preedit-attr-okuri                . preedit-attr?)
+    (skk-preedit-attr-pending-rk           . preedit-attr?)
+    (skk-preedit-attr-conv-body            . preedit-attr?)
+    (skk-preedit-attr-conv-okuri           . preedit-attr?)
+    (skk-preedit-attr-direct-pending-rk    . preedit-attr?)
+    (skk-preedit-attr-child-beginning-mark . preedit-attr?)
+    (skk-preedit-attr-child-end-mark       . preedit-attr?)
+    (skk-preedit-attr-child-committed      . preedit-attr?)
+    (skk-child-context-beginning-mark      . string?)
+    (skk-child-context-end-mark            . string?)
+    (skk-show-cursor-on-preedit?           . boolean?)
+    (skk-show-candidates-with-okuri?       . boolean?)))
+;; predefined styles
+(define skk-style-uim
+  '((skk-preedit-attr-mode-mark            . preedit-reverse)
+    (skk-preedit-attr-head                 . preedit-reverse)
+    (skk-preedit-attr-okuri                . preedit-reverse)
+    (skk-preedit-attr-pending-rk           . preedit-reverse)
+    (skk-preedit-attr-conv-body            . preedit-reverse)
+    (skk-preedit-attr-conv-okuri           . preedit-reverse)
+    (skk-preedit-attr-direct-pending-rk    . preedit-underline)
+    (skk-preedit-attr-child-beginning-mark . preedit-reverse)
+    (skk-preedit-attr-child-end-mark       . preedit-reverse)
+    (skk-preedit-attr-child-committed      . preedit-reverse)
+    (skk-child-context-beginning-mark      . "[")
+    (skk-child-context-end-mark            . "]")
+    (skk-show-cursor-on-preedit?           . #f)
+    (skk-show-candidates-with-okuri?       . #t)))
+(define skk-style-ddskk-like
+  '((skk-preedit-attr-mode-mark            . preedit-underline)
+    (skk-preedit-attr-head                 . preedit-underline)
+    (skk-preedit-attr-okuri                . preedit-underline)
+    (skk-preedit-attr-pending-rk           . preedit-underline)
+    (skk-preedit-attr-conv-body            . preedit-reverse)
+    (skk-preedit-attr-conv-okuri           . preedit-underline)
+    (skk-preedit-attr-direct-pending-rk    . preedit-underline)
+    (skk-preedit-attr-child-beginning-mark . preedit-underline)
+    (skk-preedit-attr-child-end-mark       . preedit-underline)
+    (skk-preedit-attr-child-committed      . preedit-underline)
+    (skk-child-context-beginning-mark      . "")
+    (skk-child-context-end-mark            . "")
+    (skk-show-cursor-on-preedit?           . #t)
+    (skk-show-candidates-with-okuri?       . #f)))
 
 ;; access
 (define skk-context-latin-conv
@@ -185,6 +255,20 @@
 ;; state kana head okuri tail candidates nth rk
 
 
+(define skk-setup-style
+  (lambda (style-spec style)
+    (let* ((elem (car style))
+	   (name (car elem))
+	   (val (if (symbol? (cdr elem))
+		    (symbol-value (cdr elem))
+		    (cdr elem)))
+	   (spec (assq name style-spec))
+	   (valid? (symbol-value (cdr spec))))
+      (if (valid? val)
+	  (set-symbol-value! name val))
+      (if (not (null? (cdr style)))
+	  (skk-setup-style style-spec (cdr style))))))
+
 (define skk-find-root-context
   (lambda (sc)
     (let ((pc (skk-context-parent-context sc)))
@@ -264,13 +348,28 @@
       (set! s (not s))
       (skk-context-set-kana-mode! sc s))))
 
-(define skk-get-string-by-mode
+(define skk-get-string-mode-part
   (lambda (sc res)
-    (if res
-	(if (skk-context-kana-mode sc)
-	    (car res)
-	    (cdr res))
-	nil)))
+    (if (skk-context-kana-mode sc)
+	(car res)
+	(cdr res))))
+
+(define skk-do-get-string-by-mode
+  (lambda (sc str)
+    (if str
+	(if (string? (car str))
+	    (skk-get-string-mode-part sc str)
+	    (string-append
+	     (skk-do-get-string-by-mode sc(car str))
+	     (skk-do-get-string-by-mode sc(cdr str))))
+	"")))
+
+(define skk-get-string-by-mode
+  (lambda (sc str)
+    (let ((res (skk-do-get-string-by-mode sc str)))
+      (if (and res (> (length res) 0))
+	  res
+	  nil))))
 
 (define skk-get-nth-candidate
   (lambda (sc n)
@@ -290,11 +389,12 @@
 
 (define skk-commit-raw
   (lambda (sc id key key-state)
-    (if (skk-context-parent-context sc)
-	(skk-editor-commit-raw
-	 (skk-context-editor
-	  (skk-context-parent-context sc)) key key-state)
-	(im-commit-raw id))))
+    (let ((psc (skk-context-parent-context sc)))
+      (if psc
+	  (skk-editor-commit-raw
+	   (skk-context-editor psc)
+	   key key-state)
+	  (im-commit-raw id)))))
 
 ;; commit string
 (define skk-commit
@@ -303,7 +403,7 @@
       (if psc
 	  (skk-editor-commit
 	   (skk-context-editor psc) str)
-	  (im-commit id str)))))
+	  (im-commit id (skk-lib-remove-annotation str))))))
 
 (define skk-append-string
   (lambda (sc str)
@@ -339,7 +439,9 @@
 	    (skk-context-set-nth! sc 0)
 	    (skk-context-set-state!
 	     sc 'skk-state-converting))
-	  (skk-flush sc))
+	  (if skk-use-recursive-learning?
+	      (skk-setup-child-context sc id)
+	      (skk-flush sc)))
       ())))
 
 (define skk-do-update-preedit
@@ -347,48 +449,86 @@
     (let ((rkc (skk-context-rk-context sc))
 	  (stat (skk-context-state sc))
 	  (csc (skk-context-child-context sc)))
-      (if (= stat 'skk-state-kanji)
-	  (im-pushback-preedit id preedit-reverse ""))
+      (if (and
+	   (not csc)
+	   (or
+	    (= stat 'skk-state-kanji)
+	    (= stat 'skk-state-okuri)))
+	  (im-pushback-preedit id skk-preedit-attr-mode-mark ""))
       (if (or
-	   (= stat 'skk-state-okuri)
+	   csc
 	   (= stat 'skk-state-converting))
-	  (im-pushback-preedit id preedit-reverse ""))
+	  (im-pushback-preedit id skk-preedit-attr-mode-mark ""))
       (if (or
 	   (= stat 'skk-state-kanji)
 	   (= stat 'skk-state-okuri)
-	   (skk-context-child-context sc))
+	   (and
+	    csc
+	    (= stat 'skk-state-converting)))
 	  (let ((h (skk-make-string 
 		    (skk-context-head sc)
 		    (skk-context-kana-mode sc))))
 	    (if (string? h)
 		(im-pushback-preedit
-		 id preedit-reverse
+		 id skk-preedit-attr-head
 		 h))))
       (if (and
 	   (= stat 'skk-state-converting)
-	   (not (skk-context-child-context sc)))
+	   (not csc))
 	  (begin
 	    (im-pushback-preedit
-	     id preedit-reverse
-	     (skk-get-current-candidate sc))))
-      (if (= stat 'skk-state-okuri)
-          (im-pushback-preedit id preedit-reverse "*"))
-      (im-pushback-preedit
-       id preedit-reverse
-       (skk-make-string (skk-context-okuri sc)
-			(skk-context-kana-mode sc)))
-      (im-pushback-preedit id preedit-reverse
-                           (rk-pending rkc))
+	     id
+	     (bit-or skk-preedit-attr-conv-body
+		     preedit-cursor)
+	     (skk-get-current-candidate sc))
+	    (im-pushback-preedit
+	     id skk-preedit-attr-conv-okuri
+	     (skk-make-string (skk-context-okuri sc)
+			    (skk-context-kana-mode sc)))))
+
+      (if (or
+	   (= stat 'skk-state-okuri)
+	   (and
+	    csc
+	    (= stat 'skk-state-converting)
+	    (skk-context-okuri sc)))
+	  (begin
+	    (im-pushback-preedit 
+	     id skk-preedit-attr-okuri
+	     (string-append
+	      "*" (skk-make-string (skk-context-okuri sc)
+				   (skk-context-kana-mode sc))))))
+
+      (if (or
+	   (= stat 'skk-state-direct)
+	   (= stat 'skk-state-latin)
+	   (= stat 'skk-state-wide-latin))
+	  (begin
+	    (im-pushback-preedit id skk-preedit-attr-direct-pending-rk
+				 (rk-pending rkc))
+	    (im-pushback-preedit id preedit-cursor ""))
+	  (begin
+	    (im-pushback-preedit id skk-preedit-attr-pending-rk
+				 (rk-pending rkc))
+	    (if (and
+		 (or
+		  (= stat 'skk-state-kanji)
+		  (= stat 'skk-state-okuri))
+		 skk-show-cursor-on-preedit?)
+		(im-pushback-preedit id preedit-cursor ""))))
+
       ;; child context's preedit
       (if csc
 	  (let ((editor (skk-context-editor sc)))
-	    (im-pushback-preedit id preedit-reverse "[")
-	    (im-pushback-preedit id preedit-reverse
+	    (im-pushback-preedit id skk-preedit-attr-child-beginning-mark
+				 skk-child-context-beginning-mark)
+	    (im-pushback-preedit id skk-preedit-attr-child-committed
 				 (skk-editor-get-left-string editor))
 	    (skk-do-update-preedit id csc)
-	    (im-pushback-preedit id preedit-reverse
+	    (im-pushback-preedit id skk-preedit-attr-child-committed
 				 (skk-editor-get-right-string editor))
-	    (im-pushback-preedit id preedit-reverse "]")
+	    (im-pushback-preedit id skk-preedit-attr-child-end-mark
+				 skk-child-context-end-mark)
 	    )))))
 
 (define skk-update-mode
@@ -421,6 +561,11 @@
 (define skk-proc-state-direct-no-preedit
   (lambda (key key-state id sc rkc)
     (and
+     (if (skk-cancel-key key key-state)
+	 (begin
+	   (skk-commit-raw sc id key key-state)
+	   #f)
+	 #t)
      (if (skk-wide-latin-key key key-state)
 	 (begin
 	   (skk-context-set-state! sc 'skk-state-wide-latin)
@@ -527,7 +672,7 @@
       ;; update state
       (if (= (skk-context-state sc) 'skk-state-kanji)
 	  (if res
-	      (skk-context-set-head! sc (cons res ()))
+	      (skk-append-string sc res)
 	      (set! res ())))
       (if (= (skk-context-state sc) 'skk-state-direct)
 	  (skk-get-string-by-mode sc res)
@@ -562,7 +707,9 @@
 		       (skk-flush sc))))
 	     #f)
 	   #t)
-       (if (skk-commit-key key key-state)
+       (if (or
+	    (skk-commit-key key key-state)
+	    (skk-return-key key key-state))
 	   (begin
 	     (skk-commit sc id (skk-make-string
 				(skk-context-head sc)
@@ -570,8 +717,18 @@
 	     (skk-flush sc)
 	     (skk-context-set-state! sc 'skk-state-direct)
 	     (skk-update-mode id sc)
+	     (if (skk-return-key key key-state)
+		 (skk-proc-state-direct c key key-state))
 	     #f)
 	   #t)
+       (if (skk-context-latin-conv sc)
+          (begin
+            (if (usual-char? key)
+                (let* ((s (charcode->string key))
+                       (p (cons s s)))
+                  (skk-append-string sc p)))
+            #f)
+          #t)
        (if (and (shift-key-mask key-state)
 		(skk-context-head sc))
 	   (begin
@@ -585,14 +742,6 @@
 		   (skk-append-string sc tmp)))
 	     #t)
 	   #t)
-       (if (skk-context-latin-conv sc)
-	   (begin
-	     (if (alphabet-char? key)
-		 (let* ((s (charcode->string (to-lower-char key)))
-			(p (cons s s)))
-		   (skk-append-string sc p)))
-	     #f)
-	   #t)
        (if (skk-kana-toggle-key key key-state)
 	   (begin
             (skk-commit sc id (skk-make-string
@@ -688,6 +837,18 @@
     #f))
 
 
+(define skk-back-to-kanji-state
+  (lambda (sc id)
+    (if (skk-context-candidate-window sc)
+	(im-end-candidate id))
+    (skk-context-set-state! sc 'skk-state-kanji)
+    (skk-context-set-okuri-head! sc "")
+    (if (car (skk-context-okuri sc))
+	(skk-context-set-head! sc
+			       (cons (car (skk-context-okuri sc))
+				     (skk-context-head sc))))
+    (skk-context-set-okuri! sc ())))
+
 (define skk-proc-state-converting
   (lambda (c key key-state)
     (let ((sc (skk-current-context c))
@@ -702,20 +863,13 @@
 	   #t)
        (if (skk-cancel-key key key-state)
 	   (begin
-	     ;; back to kanji mode
-	     ()
-	     (if (skk-context-candidate-window sc)
-		 (im-end-candidate id))
-	     (skk-context-set-state! sc 'skk-state-kanji)
-	     (skk-context-set-okuri-head! sc "")
-	     (if (car (skk-context-okuri sc))
-		 (skk-context-set-head! sc
-					(cons (car (skk-context-okuri sc))
-					      (skk-context-head sc))))
-	     (skk-context-set-okuri! sc ())
+	     ;; back to kanji state
+	     (skk-back-to-kanji-state sc id)
 	     #f)
 	   #t)
-       (if (skk-commit-key key key-state)
+       (if (or
+	    (skk-commit-key key key-state)
+	    (skk-return-key key key-state))
 	   (begin
 	     (set! res (skk-get-current-candidate sc))
 	     (set!
@@ -736,6 +890,11 @@
 	     (skk-flush sc)
 	     (skk-context-set-state! sc 'skk-state-direct)
 	     (skk-update-mode id sc)
+	     (if (skk-return-key key key-state)
+		 (begin
+		   (skk-commit sc id res)
+		   (set! res ())
+		   (skk-proc-state-direct c key key-state)))
 	     #f)
 	   #t)
        (begin
@@ -788,6 +947,21 @@
 		       (skk-context-set-state! sc 'skk-state-kanji)))))
 	     #f)
 	   #t)
+       ;; committing incomplete head: conformed the behavior to ddskk
+       (if (or
+	    (skk-commit-key key key-state)
+	    (skk-return-key key key-state))
+	   (begin
+	     (skk-commit sc id (skk-make-string
+				(skk-context-head sc)
+				(skk-context-kana-mode sc)))
+	     (skk-flush sc)
+	     (skk-context-set-state! sc 'skk-state-direct)
+	     (skk-update-mode id sc)
+	     (if (skk-return-key key key-state)
+		 (skk-proc-state-direct c key key-state))
+	     #f)
+	   #t)
        (begin
 	 (set! res
 	       (rk-push-key!
@@ -876,7 +1050,8 @@
       (im-pushback-mode-list id "ѱѿ")
       (im-update-mode-list id)
       (im-update-mode id 0)
-      (skk-update-prop-list id))))
+      (skk-update-prop-list id)
+      (skk-setup-style skk-style-spec (symbol-value skk-style)))))
 
 (define skk-press-key-handler
   (lambda (id key state)
@@ -915,9 +1090,12 @@
   (lambda (id idx)
     (let* ((c (find-context id))
 	   (sc (context-data c))
-	   (cand (skk-get-nth-candidate sc idx))
-	   (okuri (skk-context-okuri sc)))
-      (if okuri
+	   (dcsc (skk-find-descendant-context sc))
+	   (cand (skk-get-nth-candidate dcsc idx))
+	   (okuri (skk-context-okuri dcsc)))	  
+      (if (and
+	   okuri
+	   skk-show-candidates-with-okuri?)
 	  (string-append cand
 			 (skk-make-string okuri t))
 	  cand))))
