@@ -122,7 +122,7 @@
        (set! mode 0))
       ((tutcode-context-katakana-mode? pc)
        (set! mode 2))
-      (#t	; `else' does not supported by uim's scheme interpreter
+      (#t	; `else' is not supported by uim's scheme interpreter
        (set! mode 1)))
     (im-update-mode id mode)))
 
@@ -138,11 +138,11 @@
       ((tutcode-off-key? key key-state)
        (rk-flush rkc)
        (tutcode-context-set-on! pc #f)
-       (tutcode-update-mode id pc))
+       (tutcode-update-prop-label id pc))
       ((tutcode-kana-toggle-key? key key-state)
        (rk-flush rkc)
        (tutcode-context-kana-toggle pc)
-       (tutcode-update-mode id pc))
+       (tutcode-update-prop-label id pc))
       ((and
          (tutcode-backspace-key? key key-state)
          (not (rk-backspace rkc)))
@@ -153,7 +153,7 @@
       ((not (member (charcode->string key) (rk-expect rkc)))
        (rk-flush rkc)
        (im-commit-raw id))
-      (#t	; `else' does not supported by uim's scheme interpreter
+      (#t	; `else' is not supported by uim's scheme interpreter
        (set! res (rk-push-key! rkc (charcode->string key)))))
     (if res
 	(im-commit id
@@ -177,7 +177,7 @@
    (tutcode-on-key? key key-state)
    (begin
      (tutcode-context-set-on! pc #t)
-     (tutcode-update-mode id pc))
+     (tutcode-update-prop-label id pc))
    (im-commit-raw id)))
 
 ;;; 줿ȤνԤ
@@ -202,7 +202,8 @@
     (im-pushback-mode-list id "TUT")
     (im-pushback-mode-list id "TUT")
     (im-update-mode-list id)
-    (im-update-mode id 0)))
+    (im-update-mode id 0)
+    (tutcode-update-prop-list id)))
 
 ;;; ⡼ɤѹ줿ȤνԤ
 ;;; @param id IMID
@@ -211,7 +212,7 @@
   (let* ((c (find-context id))
 	 (pc (context-data c))
 	 (rkc (tutcode-context-rk-context pc)))
-    (cond	; `case' does not supported by uim's scheme interpreter
+    (cond	; `case' is not supported by uim's scheme interpreter
       ((= mode 0)
        (tutcode-context-set-on! pc #f))
       ((= mode 1)
@@ -222,6 +223,63 @@
        (tutcode-context-set-katakana-mode! pc #t)))
     (rk-flush rkc)))
 
+;;; ץѥƥѹåνԤ
+;;; @param id IMID
+;;; @param message ѹå
+(define (tutcode-prop-handler id message)
+  (let* ((c (find-context id))
+	 (pc (context-data c))
+	 (rkc (tutcode-context-rk-context pc)))
+    (rk-flush rkc)
+    (cond
+      ((equal? message "prop_tutcode_raw")
+       (tutcode-context-set-on! pc #f))
+      ((equal? message "prop_tutcode_hiragana")
+       (tutcode-context-set-on! pc #t)
+       (tutcode-context-set-katakana-mode! pc #f))
+      ((equal? message "prop_tutcode_katakana")
+       (tutcode-context-set-on! pc #t)
+       (tutcode-context-set-katakana-mode! pc #t)))
+    (tutcode-update-prop-label id pc)))
+
+;;; ץѥƥ٥ιԤ
+;;; @param id IMID
+;;; @param pc ƥȥꥹ
+(define (tutcode-update-prop-label id pc)
+  (let* ((str ""))
+    (cond
+      ((not (tutcode-context-on? pc))
+       (set! str "t\tľ\n"))
+      ((tutcode-context-katakana-mode? pc)
+       (set! str "\t\n"))
+      (#t
+       (set! str "\tҤ餬\n")))
+    (tutcode-update-mode id pc)
+    (im-update-prop-label id str)
+    ; äfocus_outfocus_inȡinitΥ٥äƤޤΤ
+    ; prop-listupdateƤ
+    (tutcode-update-prop-list id)))
+
+;;; ץѥƥꥹȤ򹹿롣
+;;; @param id IMID
+(define (tutcode-update-prop-list id)
+  (let* ((c (find-context id))
+	 (pc (context-data c))
+	 (str ""))
+    (cond
+      ((not (tutcode-context-on? pc))
+       (set! str "t\tľ\n"))
+      ((tutcode-context-katakana-mode? pc)
+       (set! str "\t\n"))
+      (#t
+       (set! str "\tҤ餬\n")))
+    (set! str (string-append
+	       "branch\t" str
+	       "leaf\tt\tľ\tľϥ⡼\tprop_tutcode_raw\n"
+	       "leaf\t\tҤ餬\tҤ餬ʥ⡼\tprop_tutcode_hiragana\n"
+	       "leaf\t\t\tʥ⡼\tprop_tutcode_katakana\n"))
+    (im-update-prop-list id str)))
+
 (define (tutcode-key-release-handler id key state))
 (define (tutcode-reset-handler id))
 (define (tutcode-get-candidate-handler id idx))
@@ -241,7 +299,7 @@
  tutcode-reset-handler
  tutcode-get-candidate-handler
  tutcode-set-candidate-index-handler
- nil)
+ tutcode-prop-handler)
 
 ;;; ơ֥QWERTYܡѡ
 (define tutcode-rule
