@@ -1,4 +1,9 @@
 ;;
+(define boolean?
+  (lambda (v)
+    (or (eq? v #t)
+        (eq? v #f))))
+;;
 (define alphabet-char?
   (lambda (c)
     (and (number? c)
@@ -6,6 +11,12 @@
 	  (and (> c 64) (< c 91))
 	  (and (> c 96) (< c 123))))))
 ;;
+(define usual-char?
+  (lambda (c)
+    (and (number? c)
+	 (or
+	  (and (> c 32) (< c 127))))))
+;;
 (define numeral?
   (lambda (c)
     (and (number? c)
