@@ -5,7 +5,9 @@
 
 extern LISP sym_t;
 
-#define MAX_CONTEXT 128
+#define MAX_CONTEXT 256
+#define LIBANTHY_SO	"libanthy.so"
+#define LIBANTHYDIC_SO	"libanthydic.so"
 
 struct anthy_context;
 
@@ -19,7 +21,10 @@ struct anthy_segment_stat{
 };
 
 static struct anthy_api {
+  void *diclib, *lib;
+
   int (*init)();
+  int (*quit)();
   struct anthy_context *(*create_context)();
   void (*release_context)(struct anthy_context *);
   void (*set_string)(struct anthy_context *, char *);
@@ -38,29 +43,30 @@ static struct context {
 static int
 get_anthy_api()
 {
-  void *diclib, *lib;
-
-  diclib = dlopen("libanthydic.so", RTLD_GLOBAL |RTLD_NOW);
-  if (!diclib) {
+  api.diclib = dlopen(LIBANTHYDIC_SO, RTLD_GLOBAL |RTLD_NOW);
+  if (!api.diclib) {
 	/*   fprintf (stdout, "%s\n", dlerror());*/
     return -1;
   }
-  lib = dlopen("libanthy.so", RTLD_NOW);
-  if (!lib) {
+  api.lib = dlopen(LIBANTHY_SO, RTLD_NOW);
+  if (!api.lib) {
 	/*   fprintf (stdout, "%s\n", dlerror()); */
-    dlclose(diclib);
+    dlclose(api.diclib);
     return -1;
   }
-  api.init = dlsym(lib, "anthy_init");
-  api.create_context = dlsym(lib, "anthy_create_context");
-  api.release_context = dlsym(lib, "anthy_release_context");
-  api.set_string = dlsym(lib, "anthy_set_string");
-  api.get_stat = dlsym(lib, "anthy_get_stat");
-  api.get_segment_stat = dlsym(lib, "anthy_get_segment_stat");
-  api.get_segment = dlsym(lib, "anthy_get_segment");
-  api.resize_segment = dlsym(lib, "anthy_resize_segment");
-  api.commit_segment = dlsym(lib, "anthy_commit_segment");
-  if (api.init && api.create_context &&
+  api.init = dlsym(api.lib, "anthy_init");
+  api.quit = dlsym(api.lib, "anthy_quit");
+  api.create_context = dlsym(api.lib, "anthy_create_context");
+  api.release_context = dlsym(api.lib, "anthy_release_context");
+  api.set_string = dlsym(api.lib, "anthy_set_string");
+  api.get_stat = dlsym(api.lib, "anthy_get_stat");
+  api.get_segment_stat = dlsym(api.lib, "anthy_get_segment_stat");
+  api.get_segment = dlsym(api.lib, "anthy_get_segment");
+  api.resize_segment = dlsym(api.lib, "anthy_resize_segment");
+  api.commit_segment = dlsym(api.lib, "anthy_commit_segment");
+  if (api.init &&
+      api.quit &&
+      api.create_context &&
       api.release_context && api.set_string &&
       api.get_stat && api.get_segment_stat &&
       api.resize_segment && api.commit_segment) {
@@ -230,3 +236,17 @@ uim_init_anthy()
   init_subr_3("anthy-lib-resize-segment", resize_segment);
   init_subr_3("anthy-lib-commit-segment", commit_segment);
 }
+
+void
+uim_quit_anthy()
+{
+  if (api.quit) {
+    api.quit();
+  }
+  if (api.diclib) {
+    dlclose(api.diclib);
+  }
+  if (api.lib) {
+    dlclose(api.lib);
+  }
+}
