@@ -50,3 +50,13 @@ uim_init_prime(void)
   init_subr_0("prime-lib-init", prime_lib_init);
   init_subr_1("prime-lib-send-command", prime_send_command);
 }
+
+void
+uim_quit_prime(void)
+{
+  if(primer)
+    fclose(primer);
+
+  if(primew)
+    fclose(primew);
+}
