@@ -27,7 +27,7 @@ struct obj
 	cons;
 	struct
 	  {
-	    double data;
+	    int data;
 	  }
 	flonum;
 	struct
@@ -155,7 +155,6 @@ struct obj
 #define tc_subr_4 19
 #define tc_subr_5 20
 #define tc_subr_2n 21
-#define FO_comment 35
 #define tc_user_min 50
 #define tc_user_max 100
 
@@ -174,25 +173,6 @@ typedef LISP (*SUBR_FUNC) (void);
 
 #define TKBUFFERN 5120
 
-struct gen_readio
-  {
-    int (*getc_fcn) (void *);
-    void (*ungetc_fcn) (int, void *);
-    void *cb_argument;
-  };
-
-struct gen_printio
-  {
-    int (*putc_fcn) (int, void *);
-    int (*puts_fcn) (char *, void *);
-    void *cb_argument;
-  };
-
-#define GETC_FCN(x) (*((*x).getc_fcn))((*x).cb_argument)
-#define UNGETC_FCN(c,x) (*((*x).ungetc_fcn))(c,(*x).cb_argument)
-#define PUTC_FCN(c,x) (*((*x).putc_fcn))(c,(*x).cb_argument)
-#define PUTS_FCN(c,x) (*((*x).puts_fcn))(c,(*x).cb_argument)
-
 struct repl_hooks
   {
     void (*repl_puts) (char *);
@@ -201,49 +181,32 @@ struct repl_hooks
     void (*repl_print) (LISP);
   };
 
-void process_cla (int argc, char **argv, int warnflag);
-void print_welcome (void);
-void print_hs_1 (void);
-void print_hs_2 (void);
-long no_interrupt (long n);
-LISP get_eof_val (void);
-long repl_driver (long want_sigint, long want_init, struct repl_hooks *);
+void siod_init (int argc, char **argv, int warnflag, FILE *);
+void siod_quit (void);
+
+long repl_driver (long want_init, struct repl_hooks *);
 void set_repl_hooks (void (*puts_f) (char *),
 		     LISP (*read_f) (void),
 		     LISP (*eval_f) (LISP),
 		     void (*print_f) (LISP));
-long repl (struct repl_hooks *);
 LISP my_err (char *message, LISP x);
-LISP errswitch (void);
 char *get_c_string (LISP x);
 char *get_c_string_dim (LISP x, long *);
 char *try_get_c_string (LISP x);
 long get_c_long (LISP x);
-double get_c_double (LISP x);
-LISP lerr (LISP message, LISP x);
 
-LISP newcell (long type);
 LISP cons (LISP x, LISP y);
 LISP consp (LISP x);
 LISP car (LISP x);
 LISP cdr (LISP x);
 LISP setcar (LISP cell, LISP value);
-LISP setcdr (LISP cell, LISP value);
 LISP flocons (double x);
 LISP numberp (LISP x);
-LISP plus (LISP x, LISP y);
-LISP ltimes (LISP x, LISP y);
-LISP difference (LISP x, LISP y);
-LISP Quotient (LISP x, LISP y);
-LISP greaterp (LISP x, LISP y);
-LISP lessp (LISP x, LISP y);
-LISP eq (LISP x, LISP y);
 LISP eql (LISP x, LISP y);
 LISP symcons (char *pname, LISP vcell);
 LISP symbolp (LISP x);
 LISP symbol_boundp (LISP x, LISP env);
 LISP symbol_value (LISP x, LISP env);
-LISP cintern (char *name);
 LISP rintern (char *name);
 LISP subrcons (long type, char *name, SUBR_FUNC f);
 LISP closure (LISP env, LISP code);
@@ -252,9 +215,6 @@ void gc_protect_n (LISP * location, long n);
 void gc_protect_sym (LISP * location, char *st);
 void gc_unprotect (LISP * location);
 
-void init_storage (void);
-void init_slibu (void);
-
 void init_subr (char *name, long type, SUBR_FUNC fcn);
 void init_subr_0 (char *name, LISP (*fcn) (void));
 void init_subr_1 (char *name, LISP (*fcn) (LISP));
@@ -267,66 +227,39 @@ void init_lsubr (char *name, LISP (*fcn) (LISP));
 void init_fsubr (char *name, LISP (*fcn) (LISP, LISP));
 void init_msubr (char *name, LISP (*fcn) (LISP *, LISP *));
 
-LISP assq (LISP x, LISP alist);
 LISP delq (LISP elem, LISP l);
-void set_gc_hooks (long type,
-		   LISP (*rel) (LISP),
-		   LISP (*mark) (LISP),
-		   void (*scan) (LISP),
-		   void (*free) (LISP));
-LISP gc_relocate (LISP x);
-LISP user_gc (LISP args);
-LISP gc_status (LISP args);
 void set_eval_hooks (long type, LISP (*fcn) (LISP, LISP *, LISP *));
 LISP leval (LISP x, LISP env);
 LISP symbolconc (LISP args);
-void set_print_hooks (long type, void (*fcn) (LISP, struct gen_printio *));
-LISP lprin1g (LISP exp, struct gen_printio *f);
 LISP lprin1f (LISP exp, FILE * f);
 LISP lprint (LISP exp, LISP);
 LISP lread (LISP);
 LISP lreadtk (char *, long j);
 LISP lreadf (FILE * f);
-void set_read_hooks (char *all_set, char *end_set,
-		     LISP (*fcn1) (int, struct gen_readio *),
-		     LISP (*fcn2) (char *, long, int *));
-LISP apropos (LISP);
 LISP vload (char *fname, long cflag, long rflag);
 LISP require (LISP fname);
-LISP save_forms (LISP fname, LISP forms, LISP how);
 LISP nullp (LISP x);
 LISP strcons (long length, char *data);
-LISP cons_array (LISP dim, LISP kind);
-LISP arcons (long typecode, long n, long initp);
-LISP string_append (LISP args);
 LISP string_length (LISP string);
-LISP string_search (LISP, LISP);
-LISP substring (LISP, LISP, LISP);
 LISP string_trim (LISP);
 LISP string_trim_left (LISP);
 LISP string_trim_right (LISP);
 LISP string_upcase (LISP);
 LISP string_downcase (LISP);
-void init_subrs (void);
 //LISP copy_list_slib (LISP);
 LISP href (LISP, LISP);
 LISP hset (LISP, LISP, LISP);
 LISP equal (LISP, LISP);
-LISP assoc (LISP x, LISP alist);
 LISP make_list (LISP x, LISP v);
 void set_fatal_exit_hook (void (*fcn) (void));
-LISP parse_number (LISP x);
 LISP intern (LISP x);
 void init_trace (void);
-long repl_c_string (char *, long want_sigint, long want_init, long want_print);
-char *siod_version (void);
+long repl_c_string (char *, long want_init, long want_print);
 LISP nreverse (LISP);
 LISP number2string (LISP, LISP, LISP, LISP);
 LISP string2number (LISP, LISP);
 LISP siod_verbose (LISP);
 int siod_verbose_check (int);
-LISP setvar (LISP, LISP, LISP);
-long allocate_user_tc (void);
 LISP cadr (LISP);
 LISP caar (LISP);
 LISP cddr (LISP);
@@ -342,13 +275,9 @@ void chk_string (LISP, char **, long *);
 LISP a_true_value (void);
 LISP lapply (LISP fcn, LISP args);
 LISP mallocl (void *lplace, long size);
-void gput_st (struct gen_printio *, char *);
 void put_st (char *st);
 LISP listn (long n,...);
 char *must_malloc (unsigned long size);
-LISP lstrbreakup (LISP str, LISP lmarker);
-LISP lstrunbreakup (LISP elems, LISP lmarker);
-LISP nconc (LISP, LISP);
 LISP poparg (LISP *, LISP);
 FILE *get_c_file (LISP p, FILE * deflt);
 char *last_c_errmsg (int);
@@ -363,13 +292,7 @@ char *safe_strcpy (char *s1, size_t size1, const char *s2);
 char *safe_strcat (char *s1, size_t size1, const char *s2);
 
 size_t safe_strlen (const char *s, size_t size);
-LISP memq (LISP x, LISP il);
-LISP lstrbreakup (LISP, LISP);
-LISP lstrbreakup (LISP, LISP);
-LISP nth (LISP, LISP);
-LISP butlast (LISP);
 LISP last (LISP);
-LISP readtl (struct gen_readio *f);
 LISP funcall1 (LISP, LISP);
 LISP funcall2 (LISP, LISP, LISP);
 LISP apply1 (LISP, LISP, LISP);
@@ -378,15 +301,39 @@ LISP lungetc (LISP i, LISP p);
 LISP lputc (LISP c, LISP p);
 LISP lputs (LISP str, LISP p);
 
-int assemble_options (LISP,...);
 LISP ccall_catch (LISP tag, LISP (*fcn) (void *), void *);
 LISP lref_default (LISP li, LISP x, LISP fcn);
 
+extern LISP sym_t;
 
-LISP symalist (char *item,...);
+extern long siod_verbose_level;
+extern char *siod_lib;
 
-LISP encode_st_mode (LISP l);
-LISP encode_open_flags (LISP l);
+struct gc_protected
+  {
+    LISP *location;
+    long length;
+    struct gc_protected *next;
+  };
 
+#ifdef THINK_C
+extern int ipoll_counter;
+void full_interrupt_poll (int *counter);
+#define INTERRUPT_CHECK() if (--ipoll_counter < 0) full_interrupt_poll(&ipoll_counter)
+#else
+#define INTERRUPT_CHECK()
+#endif
+
+long no_interrupt (long n);
+void err_ctrl_c (void);
+double myruntime (void);
+void fput_st (FILE * f, char *st);
+void put_st (char *st);
+void gc_for_newcell (void);
+void gc_ms_stats_start (void);
+void gc_ms_stats_end (void);
+void gc_mark (LISP ptr);
+
+void init_subrs_a (void);
 
 #endif /* __SIOD_H__ */
