@@ -399,8 +399,6 @@ add_line_to_cache(struct dic_info *di, struct skk_line *sl)
   di->head.next = sl;
 }
 
-
-
 static struct skk_line *
 skk_search_line_from_file(struct dic_info *di, char *s, char okuri_head)
 {
@@ -573,16 +571,15 @@ reorder_candidate(struct skk_cand_array *ca, char *str)
       nth = i;
     }
   }
-  if (nth == 0) {
-    return ;
-  }
 
   /* shift array */
   tmp = ca->cands[nth];
-  for (i = nth; i > 0; i--) {
-    ca->cands[i] = ca->cands[i - 1];
+  if (nth) {
+    for (i = nth; i > 0; i--) {
+      ca->cands[i] = ca->cands[i - 1];
+    }
+    ca->cands[0] = tmp;
   }
-  ca->cands[0] = tmp;
   /**/
   if (nth >= ca->nr_real_cands) {
     ca->nr_real_cands ++;
@@ -634,7 +631,6 @@ learn_word_to_cand_array(struct skk_cand_array *ca, char *word)
   }
   if (nth == -1) {
     push_back_candidate_to_array(ca, word);
-    nth = ca->nr_cands - 1;
   }
   reorder_candidate(ca, word);
   ca->line->need_save = 1;
@@ -643,6 +639,15 @@ learn_word_to_cand_array(struct skk_cand_array *ca, char *word)
 static char *
 sanitize_word(char *arg)
 {
+  char *tmp;
+  if (!arg || !strlen(arg)) {
+    return NULL;
+  }
+  for (tmp = arg; *tmp; tmp++) {
+    if (strchr(" /[]()\n", *tmp)) {
+      return NULL;
+    }
+  }
   return strdup(arg);
 }
 
@@ -663,12 +668,15 @@ skk_learn_word(LISP head_, LISP okuri_head_, LISP okuri_, LISP word_)
   if (ca) {
     learn_word_to_cand_array(ca, word);
   }
-  if (okuri_ != NIL) {
+
+  tmp = uim_get_c_string(okuri_);
+  if (strlen(tmp)) {
     ca = find_cand_array_lisp(head_, okuri_head_, NIL, 1);
     if (ca) {
       learn_word_to_cand_array(ca, word);
     }
   }
+  free(tmp);
   free(word);
   return NIL;
 }
@@ -726,6 +734,24 @@ write_out_array(FILE *fp, struct skk_cand_array *ca)
   }
 }
 
+static void
+write_out_line(FILE *fp, struct skk_line *sl)
+{
+  struct skk_cand_array *ca;
+  int i;
+  fprintf(fp, "%s", sl->head);
+  if (sl->okuri_head) {
+    fprintf(fp, "%c /", sl->okuri_head);
+  } else {
+    fprintf(fp, " /");
+  }
+  for (i = 0; i < sl->nr_cand_array; i++) {
+    ca = &sl->cands[i];
+    write_out_array(fp, ca);
+  }
+  fprintf(fp, "\n");
+}
+
 static LISP
 skk_lib_save_personal_dictionary(LISP fn_)
 {
@@ -733,26 +759,20 @@ skk_lib_save_personal_dictionary(LISP fn_)
   char *fn = uim_get_c_string(fn_);
   struct skk_line *sl;
 
-  fp = fopen(fn, "w");
-  free(fn);
+  if (fn) {
+    fp = fopen(fn, "w");
+    free(fn);
+  } else {
+    fp = stdout;
+  }
   if (!fp) {
     return NIL;
   }
 
   for (sl = skk_dic->head.next; sl; sl = sl->next) {
-    struct skk_cand_array *ca;
-    int i;
-    fprintf(fp, "%s", sl->head);
-    if (sl->okuri_head) {
-      fprintf(fp, "%c /", sl->okuri_head);
-    } else {
-      fprintf(fp, " /");
+    if (sl->need_save) {
+      write_out_line(fp, sl);
     }
-    for (i = 0; i < sl->nr_cand_array; i++) {
-      ca = &sl->cands[i];
-      write_out_array(fp, ca);
-    }
-    fprintf(fp, "\n");
   }
   fclose(fp);
   return NIL;
@@ -790,34 +810,21 @@ skk_lib_read_personal_dictionary(LISP fn_)
 }
 
 static LISP
-skk_split_string(LISP str_)
+skk_lib_remove_annotation(LISP str_)
 {
-  char *str = get_c_string(str_);
-  unsigned char *cur = str;
-  LISP res = NIL;
-  while (*cur) {
-    LISP tmp;
-    char buf[3];
-    int len;
-    buf[2] = 0;
-    if (*cur > 127) {
-      /* 2 bytes */
-      buf[0] = cur[0];
-      buf[1] = cur[1];
-      len = 2;
-      cur ++;
-    } else {
-      buf[0] = cur[0];
-      buf[1] = 0;
-      len = 1;
-    }
-    res = cons (strcons(len, buf), res);
-    cur ++;
+  char *str = uim_get_c_string(str_);
+  char *sep = strrchr(str, ';');
+  LISP res;
+  if (sep) {
+    *sep = 0;
   }
+  res = strcons(strlen(str), str);
+  free(str);
   return res;
 }
 
-void uim_init_skk_dic()
+void
+uim_init_skk_dic(void)
 {
   init_subr_1("skk-lib-dic-open", skk_dic_open);
   init_subr_1("skk-lib-read-personal-dictionary", skk_lib_read_personal_dictionary);
@@ -827,5 +834,22 @@ void uim_init_skk_dic()
   init_subr_3("skk-lib-get-nr-candidates", skk_get_nr_candidates);
   init_subr_4("skk-lib-commit-candidate", skk_commit_candidate);
   init_subr_4("skk-lib-learn-word", skk_learn_word);
-  init_subr_1("skk-lib-split-string", skk_split_string);
+  init_subr_1("skk-lib-remove-annotation", skk_lib_remove_annotation);
+}
+
+void
+uim_quit_skk_dic(void)
+{
+  struct skk_line *sl, *tmp;
+  if (!skk_dic) {
+    return ;
+  }
+  munmap(skk_dic->addr, skk_dic->size);
+  sl = skk_dic->head.next;
+  while (sl) {
+    tmp = sl;
+    sl = sl->next;
+    free_skk_line(tmp);
+  }
+  free(skk_dic);
 }
