@@ -160,7 +160,7 @@ uim_schedule_cb(uim_context uc, int type, char *str, int n1, int n2)
 void
 uim_eval_string(uim_context uc, char *buf)
 {
-  repl_c_string(buf,0,0,1);
+  repl_c_string(buf,0,1);
   if (!uc->cb_q.flushing) {
     uc->cb_q.flushing ++;
     uim_flush_cb(uc);
