@@ -36,14 +36,17 @@ uim_helper_send_message(int fd, char *message)
      so we should read here and proc such condition. */
 
   //  rfile = fdopen(fd, "r");
-  wfile = fdopen(fd, "w");
-  
-  fputs(message, wfile);
-  fputs("\n",wfile);
-  fflush(wfile);
-  /*  printf("send massage:%s",message);*/
-  return "";
 
+  if(uim_helper_fd(fd, WRITE)){
+    wfile = fdopen(fd, "w");
+    
+    fputs(message, wfile);
+    fputs("\n",wfile);
+    fflush(wfile);
+
+  }
+    /*  printf("send massage:%s",message);*/
+    return "";
 }
 
 char *
@@ -89,7 +92,7 @@ uim_helper_fd(int fd, enum RorW rw)
     if (rc < 0)
       return -1;
     
-    return FD_ISSET(fd,&fds) ? 1 : 0;
+    return FD_ISSET(fd, &fds) ? 1 : 0;
 }
 
 int uim_helpler_fd_readable(int fd)
