@@ -32,6 +32,9 @@ static struct {
   {UKey_F11, "F11"},
   {UKey_F12, "F12"},
   {UKey_Multi_key, "Multi_key"},
+  {UKey_Mode_switch, "Mode_switch"},
+  {UKey_Henkan_Mode, "Henkan_Mode"},
+  {UKey_Muhenkan, "Muhenkan"},
   {UKey_Shift_key, "Shift_key"},
   {UKey_Alt_key, "Alt_key"},
   {UKey_Control_key, "Control_key"},
@@ -92,6 +95,7 @@ handle_key(uim_context uc, char *p, int key, int state)
 
   rv = keycode_to_sym(key, keybuf);
   if (rv == -1) {
+    uc->commit_raw_flag = 1;
     return;
   }
   snprintf(buf, 99, "(key-%s-handler %d %s %d)", p,
