@@ -57,7 +57,7 @@ uim_get_c_string(LISP str)
   long len;
   char *buf;
   s = get_c_string_dim(str, &len);
-  buf = (char *)malloc(len + 1);
+  buf = (char *)malloc(sizeof(char)*(len + 1));
   strncpy(buf, s, len);
   buf[len] = 0;
   return buf;
@@ -205,7 +205,7 @@ uim_strsplit(char *splittee, char *splitter)
       tmp ++;
     }
     /* store */
-    res[i] = malloc(sizeof(char *) * (len + 1));
+    res[i] = malloc(sizeof(char) * (len + 1));
     strncpy(res[i], cur, len);
     res[i][len] = 0;
     cur = tmp;
@@ -250,6 +250,76 @@ uim_split_string(LISP _splittee, LISP _splitter)
    return l;
 }
 
+static LISP
+string_to_list(LISP str_)
+{
+  char *str = get_c_string(str_);
+  unsigned char *cur = str;
+  LISP res = NIL;
+  while (*cur) {
+    LISP tmp;
+    char buf[3];
+    int len;
+    buf[2] = 0;
+    if (*cur > 127) {
+      /* 2 bytes */
+      buf[0] = cur[0];
+      buf[1] = cur[1];
+      len = 2;
+      cur ++;
+    } else {
+      buf[0] = cur[0];
+      buf[1] = 0;
+      len = 1;
+    }
+    res = cons (strcons(len, buf), res);
+    cur ++;
+  }
+  return res;
+}
+
+static LISP
+uim_symbol_value(const char *symbol_str)
+{
+  LISP symbol_str_ = rintern(symbol_str);
+  
+  if(symbol_boundp(symbol_str_, NIL) == sym_t) {
+    return symbol_value(symbol_str_, NIL);         
+  } else {
+    return NIL;
+  }
+}
+
+char *
+uim_symbol_value_str(const char *symbol_str)
+{
+  LISP val_;
+  char *val;
+  val_ = uim_symbol_value(symbol_str);
+
+  if( val_ != NIL) {
+    val = uim_get_c_string(val_);
+  } else {
+    val = NULL;
+  }
+  return val;
+}
+
+int
+uim_symbol_value_int(const char *symbol_str)
+{
+  LISP val_;
+  int val;
+  val_ = uim_symbol_value(symbol_str);
+
+  if( val_ != NIL) {
+    val = get_c_long(val_);
+  } else {
+    val = NULL;
+  }
+  return val;
+}
+
 void
 uim_init_util_subrs()
 {
@@ -263,5 +333,5 @@ uim_init_util_subrs()
   init_subr_2("rk-lib-expect-seq", rk_expect_seq);
   init_subr_1("getenv", c_getenv);
   init_subr_2("string-split", uim_split_string);
-
+  init_subr_1("string-to-list", string_to_list);
 }
