@@ -26,7 +26,7 @@ int g_option_mask;
 int scr_width,scr_height;
 int host_byte_order;
 
-static int lib_uim_fd;
+int lib_uim_fd;
 static char *result;
 
 #define VERSION_NAME "uim-xim under the way! Version "PACKAGE_VERSION"\n"
@@ -135,8 +135,8 @@ add_window_watch(Window id, WindowIf *w, int mask)
     std::pair<unsigned int,WindowIf *> p(id, w);
     window_watch_stat.insert(p);
 
-    //٥ȥޥX.hƤΤƱͤĤ
-    //˰¸ץϽ񤫤ʤ
+    //Event mash has same value defined in X,
+    //but do not depend on.
     int emask = 0;
     if (mask & EXPOSE_MASK) {
 	emask |= ExposureMask;
@@ -360,21 +360,12 @@ helper_str_parse(char *str)
 static void
 helper_read_cb(int fd, int ev)
 {
-  char *tmp;
-
-  tmp = uim_helper_read_proc(fd);
-
-  if(!result)
-    result = strdup("");
-
-  result = (char*)realloc(result, strlen(result) + strlen(tmp)+1);
-  strcat(result, tmp);
-
-  if(uim_helper_str_terminated(result)) {
-    helper_str_parse(result);
-    free(result);
-    result = strdup("");
-  }
+    uim_helper_read_proc(fd);
+    char *tmp;
+    while ((tmp = uim_helper_get_message())) {
+	helper_str_parse(tmp);
+	free(tmp);
+    }
 }
 
 static void
@@ -417,19 +408,20 @@ main(int argc, char **argv)
 					NULL, NULL);
     int nr = uim_get_nr_im(uc);
     for (int i = 0; i < nr; i++) {
-	char *name = uim_get_im_name(uc, i);
-	char *lang = uim_get_im_language(uc, i);
-	if (!XimServer::isSupportedLanguage(lang)) {
+	char *name = (char *)uim_get_im_name(uc, i);
+	char *lang = (char *)uim_get_im_language(uc, i);
+	Locale *lc = getLocale(lang);
+	if (!lc) {
 	    continue;
 	}
-	XimServer *xs = new XimServer(name, lang);
+	XimServer *xs = new XimServer(lc, name, lang);
 	bool res = xs->setupConnection();
 	if (!res) {
 	    delete xs;
 	}
 	printf("XMODIFIERS=@im=uim-%s ,(%s) registered\n", name, lang);
     }
-	
+    
     connection_setup();
     if (pretrans_setup() == -1) {
 	return 0;
