@@ -18,15 +18,20 @@
 # endif
 #endif
 
-extern input_style input_style_tab[];
-
 char invalid_style_msg[]=
-"ݡȤƤʤϥ\n"
-"򥯥饤Ȥ׵ᤷƤޤ";
+"Client requested unsupported input style";
 
 XimIC *XimIC::current_ic;
 int XimIC::nrActiveIC;
 
+struct FSCache{
+    int refc;
+    XFontSet fs;
+    char *name;
+};
+
+static std::list<FSCache> fs_cache;
+
 static void
 ui_update_ic_stat()
 {
@@ -37,6 +42,46 @@ ui_update_input_mode(int mode)
 {
 }
 
+static XFontSet
+get_font_set(char *name)
+{
+    std::list<FSCache>::iterator it;
+    for (it = fs_cache.begin(); it != fs_cache.end(); it++) {
+	if (!strcmp(it->name, name)) {
+	    it->refc++;
+	    return it->fs;
+	}
+    }
+    struct FSCache fc;
+    char **missing,*def;
+    int nr_missing;
+    fc.fs = XCreateFontSet(XimServer::gDpy,name,
+			   &missing,&nr_missing,&def);
+    if (missing) {
+	XFreeStringList(missing);
+    }
+    fc.refc = 1;
+    fc.name = strdup(name);
+    fs_cache.push_back(fc);
+    return fc.fs;
+}
+
+static void
+release_font_set(char *name)
+{
+    std::list<FSCache>::iterator it;
+    for (it = fs_cache.begin(); it != fs_cache.end(); it++) {
+	if (!strcmp(it->name, name)) {
+	    it->refc--;
+	    if (!it->refc) {
+		XFreeFontSet(XimServer::gDpy, it->fs);
+		fs_cache.erase(it);
+	    }
+	    return ;
+	}
+    }
+}
+
 pe_stat::pe_stat(InputContext *c)
 {
     cont = c;
@@ -88,8 +133,8 @@ icxatr::icxatr()
 icxatr::~icxatr()
 {
     if (font_set_name) {
+	release_font_set(font_set_name);
 	free((void *)font_set_name);
-	XFreeFontSet(XimServer::gDpy,font_set);
     }
 }
 
@@ -135,13 +180,7 @@ void icxatr::set_atr(int id, C8 *val, int len, int o)
 	}
 	font_set_name = strdup(new_fsn);
 
-	char **missing,*def;
-	int nr_missing;
-	font_set = XCreateFontSet(XimServer::gDpy,font_set_name,
-				  &missing,&nr_missing,&def);
-	if (missing) {
-	    XFreeStringList(missing);
-	}
+	font_set = get_font_set(font_set_name);
     }
     break;
     case ICA_Area:
@@ -297,6 +336,7 @@ void XimIC::setFocus()
     nrActiveIC ++;
     ui_update_ic_stat();
     ui_update_input_mode(m_kkContext->getMode());
+    m_kkContext->focusIn();
     if (mConvdisp) {
 	mConvdisp->set_focus();
     }
@@ -313,6 +353,7 @@ void XimIC::unsetFocus()
 	ui_update_ic_stat();
     }
 
+    m_kkContext->focusOut();
     if (mConvdisp) {
 	mConvdisp->unset_focus();
     }
@@ -370,7 +411,7 @@ void XimIC::send_key_event(XKeyEvent *e)
 void XimIC::commit_string(char *str)
 {
     uString us;
-    str_to_ustring(&us, str);
+    mConn->getXimServer()->strToUstring(&us, str);
     append_ustring(&mPending, &us);
 }
 
@@ -432,9 +473,10 @@ int XimIC::get_ic_atr(int id, TxPacket *t)
 int XimIC::lookup_style(unsigned long s)
 {
     int i;
-    for (i = 0; input_style_tab[i].x_style; i++) {
-        if (input_style_tab[i].x_style == (int)s) {
-            return input_style_tab[i].style;
+    struct input_style *is = mConn->getXimServer()->getInputStyles();
+    for (i = 0; is[i].x_style; i++) {
+        if (is[i].x_style == (int)s) {
+            return is[i].style;
         }
     }
     return IS_INVALID;
