@@ -12,35 +12,32 @@
 # endif
 #endif
 
+extern int lib_uim_fd;
 extern Atom xim_servers;
 
-static char *
-ustring_to_str(uString *s)
-{
-    uString::iterator i;
-    int l=0;
-    char *c;
-    for (i = s->begin(); i!= s->end(); i++) {
-        l++;
-        if (*i > 256) {
-            l ++;
-        }
-    }
-    c = (char *)malloc(l+1);
-    c[l] = 0;
-    l = 0;
-    for (i = s->begin() ; i!= s->end(); i++) {
-        if (*i < 256) {
-            c[l] = *i;
-            l++;
-        } else {
-            c[l] = ((*i)>>8)|0x80;
-            c[l+1] = ((*i)&0xff)|0x80;
-            l+=2;
-        }
-    }
-    return c;
-}
+// tables
+static input_style input_style_tab_with_over_the_spot[] = {
+    {XIMPreeditNothing|XIMStatusNothing, IS_ROOT_WINDOW},
+    //{XIMPreeditPosition|XIMStatusArea, IS_OVER_THE_SPOT},// emacs
+    {XIMPreeditPosition|XIMStatusNothing, IS_OVER_THE_SPOT},
+    //{XIMPreeditCallbacks|XIMStatusCallbacks, IS_ON_THE_SPOT},// OOo
+    //{XIMPreeditArea|XIMStatusArea, IS_ROOT_WINDOW},
+    {XIMPreeditCallbacks|XIMStatusNothing, IS_ON_THE_SPOT},
+    {0, 0},
+};
+static input_style input_style_tab_without_over_the_spot[] = {
+    {XIMPreeditNothing|XIMStatusNothing, IS_ROOT_WINDOW},
+    //{XIMPreeditPosition|XIMStatusArea, IS_OVER_THE_SPOT},// emacs
+    //{XIMPreeditPosition|XIMStatusNothing, IS_OVER_THE_SPOT},
+    //{XIMPreeditCallbacks|XIMStatusCallbacks, IS_ON_THE_SPOT},// OOo
+    //{XIMPreeditArea|XIMStatusArea, IS_ROOT_WINDOW},
+    {XIMPreeditCallbacks|XIMStatusNothing, IS_ON_THE_SPOT},
+    {0, 0},
+};
+// XIMPreeditArea,XIMPreeditCallbacks,XIMPreeditPosition
+// XIMPreeditNothing,XIMPreeditNone
+// XIMStatusArea,XIMStatusCallbacks
+// XIMStatusNothing,XIMStatusNone
 
 void print_ustring(uString *s)
 {
@@ -76,7 +73,6 @@ void append_ustring(uString *d, uString *s)
     }
 }
 
-// EUCJISξб
 void str_to_ustring(uString *d, char *s)
 {
     int i,len;
@@ -102,10 +98,11 @@ void str_to_ustring(uString *d, char *s)
     }
 }
 
-XimServer::XimServer(char *name, char *lang)
+XimServer::XimServer(Locale *lc, char *name, char *lang)
 {
     mName = name;
     mLang = lang;
+    mLocale = lc;
 }
 
 InputContext *XimServer::createContext(XimIC *xic)
@@ -121,10 +118,26 @@ InputContext *XimServer::createContext(XimIC *xic)
                          InputContext::candidate_begin_cb,
                          InputContext::candidate_update_cb,
                          InputContext::candidate_end_cb);
+    uim_set_prop_list_update_cb(uc,
+				InputContext::update_prop_list_cb);
+    uim_set_prop_label_update_cb(uc,
+				 InputContext::update_prop_label_cb);
+
+    uim_prop_list_update(uc);
+    
     ic->setUC(uc);
     return ic;
 }
 
+struct input_style *
+XimServer::getInputStyles()
+{
+    if (mLocale->supportOverTheSpot()) {
+	return input_style_tab_with_over_the_spot;
+    }
+    return input_style_tab_without_over_the_spot;
+}
+
 bool
 XimServer::setupConnection()
 {
@@ -184,14 +197,33 @@ XimServer::setupConnection()
 char *
 XimServer::uStringToCtext(uString *us)
 {
-    char *str = ustring_to_str(us);
-    XTextProperty prop;
-    XmbTextListToTextProperty(XimServer::gDpy, &str, 1,
-			      XCompoundTextStyle, &prop);
-    free(str);
-    char *res = strdup((char *)prop.value);
-    XFree(prop.value);
-    return res;
+    return mLocale->uStringToCtext(us);
+}
+
+void
+XimServer::strToUstring(uString *d, char *s)
+{
+    int i,len;
+    bool inJis= false;
+    len = strlen(s);
+    for (i = 0; i < len; i++) {
+	if (s[i] == 0x1b) {
+	    if (s[i+1] == 0x24 && s[i+2]== 0x42) {
+		inJis = true;
+	    } else if (s[i+1] == 0x28 && s[i+2] == 0x42) {
+		inJis = false;
+	    }
+	    i += 2;
+	} else if(inJis || (s[i] & 0x80)) {
+	    //2Хʸ
+	    uchar ch;
+	    ch = ((s[i]&0x7f)<<8)|(s[i+1]&0x7f);
+	    i++;
+	    d->push_back(ch);
+	} else {
+	    d->push_back(s[i]);
+	}
+    }
 }
 
 XimServer *XimServer::findServer(Window w)
@@ -204,14 +236,6 @@ XimServer *XimServer::findServer(Window w)
     return it->second;
 }
 
-bool XimServer::isSupportedLanguage(char *lang)
-{
-    if (!strcmp(lang, "ja")) {
-	return true;
-    }
-    return false;
-}
-
 //
 // InputContextΥ᥽å
 InputContext::InputContext(XimServer *svr, XimIC *ic)
@@ -232,6 +256,19 @@ InputContext::~InputContext()
     uim_release_context(mUc);
 }
 
+void
+InputContext::focusIn()
+{
+    uim_prop_list_update(mUc);	
+    uim_helper_client_focus_in(mUc);
+}
+
+void
+InputContext::focusOut()
+{
+    uim_helper_client_focus_out(mUc);
+}
+
 XimServer *
 InputContext::getServer()
 {
@@ -291,6 +328,18 @@ void InputContext::candidate_end_cb(void *ptr)
     ic->candidate_end();
 }
 
+void InputContext::update_prop_list_cb(void *ptr, char *str)
+{
+    InputContext *ic = (InputContext *)ptr;
+    ic->update_prop_list(str);
+}
+
+void InputContext::update_prop_label_cb(void *ptr, char *str)
+{
+    InputContext *ic = (InputContext *)ptr;
+    ic->update_prop_label(str);
+}
+
 void InputContext::clear_preedit()
 {
     m_pe->clear();
@@ -313,7 +362,7 @@ void InputContext::pushback_preedit_string(int attr, char *str)
     }
     m_pe->new_segment(p);
     uString js;
-    str_to_ustring(&js, str);
+    mServer->strToUstring(&js, str);
 
     uString::iterator it;
     for (it = js.begin(); it != js.end(); it++) {
@@ -405,6 +454,32 @@ void InputContext::candidate_end()
     disp->end();
 }
 
+void InputContext::update_prop_list(char* str)
+{
+    char tmp[BUFSIZ];
+    int len;
+
+    len = snprintf(tmp, sizeof(tmp),
+		   "prop_list_update\ncharset=EUC-JP\n%s", str);
+
+    if (len != -1 && len <= sizeof(tmp)) {
+	uim_helper_send_message(lib_uim_fd, tmp);
+    }
+}
+
+void InputContext::update_prop_label(char* str)
+{
+    char tmp[BUFSIZ];
+    int len;
+
+    len = snprintf(tmp, sizeof(tmp),
+		   "prop_label_update\ncharset=EUC-JP\n%s", str);
+
+    if (len != -1 && len <= sizeof(tmp)) {
+	uim_helper_send_message(lib_uim_fd, tmp);
+    }
+}
+
 keyState::keyState(keyEventX *x)
 {
     mModifier = 0;
@@ -451,6 +526,9 @@ keyState::keyState(keyEventX *x)
 	case XK_F11: mKey = UKey_F11; break;
 	case XK_F12: mKey = UKey_F12; break;
 	case XK_Multi_key: mKey = UKey_Multi_key; break;
+	case XK_Mode_switch: mKey = UKey_Mode_switch; break;
+	case XK_Henkan_Mode: mKey = UKey_Henkan_Mode; break;
+	case XK_Muhenkan: mKey = UKey_Muhenkan; break;
 	default: mKey = UKey_Other;
 	}
     }
