@@ -117,6 +117,8 @@ public:
     void setMode(int mode);
     uString * clear();
     void setConvdisp(Convdisp *);
+    void focusIn();
+    void focusOut();
     XimIC *get_ic();
     XimServer *getServer();
     bool extra_input(char *s);
@@ -126,6 +128,8 @@ public:
     void candidate_begin(int nr, int index);
     void candidate_update(int index);
     void candidate_end();
+    void update_prop_list(char *str);
+    void update_prop_label(char *str);
 public:
     static void commit_cb(void *, char *);
     static void clear_cb(void *);
@@ -134,6 +138,8 @@ public:
     static void candidate_begin_cb(void *ptr, int nr, int index);
     static void candidate_update_cb(void *ptr, int index);
     static void candidate_end_cb(void *ptr);
+    static void update_prop_list_cb(void *ptr, char *str);
+    static void update_prop_label_cb(void *ptr, char *str);
 protected:
     void commit_string(char *);
     XimIC *mXic;
@@ -143,29 +149,36 @@ protected:
     uim_context mUc;
 };
 
-//ĤѴ󥸥ɽ魯e.g.SKK,Canna,Wnn etc
+class Locale {
+public:
+    virtual char *uStringToCtext(uString *us);
+    virtual bool supportOverTheSpot();
+};
+
+Locale *getLocale(char *lang);
+
 class XimServer {
 public:
-    XimServer(char *name, char *lang);
+    XimServer(Locale *lc, char *name, char *lang);
     ~XimServer(){};
     InputContext *createContext(XimIC *);
 
     bool setupConnection();
     void setupNewConnection(XClientMessageEvent *ev);
     char *uStringToCtext(uString *js);
+    void strToUstring(uString *d, char *s);
+    struct input_style *getInputStyles();
 public:
     static XimServer *findServer(Window w);
-    static bool isSupportedLanguage(char *);
     static Display *gDpy;
     static std::map<Window , XimServer *>gServerMap;
 private:
+    Locale *mLocale;
     Window mSelectionWin;
     Atom mServerAtom;
     char *mName, *mLang;
 };
 
-char *string_to_ctext(char *s);
-
 #endif
 /*
  * Local variables:
